# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import SettingsTab, translate, Receiver
from openlp.core.ui import AlertLocation
from openlp.core.lib.ui import UiStrings, create_valign_selection_widgets
from openlp.core.lib.settings import Settings

class AlertsTab(SettingsTab):
    """
    AlertsTab is the alerts settings tab in the settings dialog.
    """
    def __init__(self, parent, name, visible_title, icon_path):
        SettingsTab.__init__(self, parent, name, visible_title, icon_path)

    def setupUi(self):
        self.setObjectName(u'AlertsTab')
        SettingsTab.setupUi(self)
        self.fontGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.fontGroupBox.setObjectName(u'fontGroupBox')
        self.fontLayout = QtGui.QFormLayout(self.fontGroupBox)
        self.fontLayout.setObjectName(u'fontLayout')
        self.fontLabel = QtGui.QLabel(self.fontGroupBox)
        self.fontLabel.setObjectName(u'fontLabel')
        self.fontComboBox = QtGui.QFontComboBox(self.fontGroupBox)
        self.fontComboBox.setObjectName(u'fontComboBox')
        self.fontLayout.addRow(self.fontLabel, self.fontComboBox)
        self.fontColorLabel = QtGui.QLabel(self.fontGroupBox)
        self.fontColorLabel.setObjectName(u'fontColorLabel')
        self.colorLayout = QtGui.QHBoxLayout()
        self.colorLayout.setObjectName(u'colorLayout')
        self.fontColorButton = QtGui.QPushButton(self.fontGroupBox)
        self.fontColorButton.setObjectName(u'fontColorButton')
        self.colorLayout.addWidget(self.fontColorButton)
        self.colorLayout.addSpacing(20)
        self.backgroundColorLabel = QtGui.QLabel(self.fontGroupBox)
        self.backgroundColorLabel.setObjectName(u'backgroundColorLabel')
        self.colorLayout.addWidget(self.backgroundColorLabel)
        self.backgroundColorButton = QtGui.QPushButton(self.fontGroupBox)
        self.backgroundColorButton.setObjectName(u'backgroundColorButton')
        self.colorLayout.addWidget(self.backgroundColorButton)
        self.fontLayout.addRow(self.fontColorLabel, self.colorLayout)
        self.fontSizeLabel = QtGui.QLabel(self.fontGroupBox)
        self.fontSizeLabel.setObjectName(u'fontSizeLabel')
        self.fontSizeSpinBox = QtGui.QSpinBox(self.fontGroupBox)
        self.fontSizeSpinBox.setObjectName(u'fontSizeSpinBox')
        self.fontLayout.addRow(self.fontSizeLabel, self.fontSizeSpinBox)
        self.timeoutLabel = QtGui.QLabel(self.fontGroupBox)
        self.timeoutLabel.setObjectName(u'timeoutLabel')
        self.timeoutSpinBox = QtGui.QSpinBox(self.fontGroupBox)
        self.timeoutSpinBox.setMaximum(180)
        self.timeoutSpinBox.setObjectName(u'timeoutSpinBox')
        self.fontLayout.addRow(self.timeoutLabel, self.timeoutSpinBox)
        self.verticalLabel, self.verticalComboBox = \
            create_valign_selection_widgets(self.fontGroupBox)
        self.verticalLabel.setObjectName(u'verticalLabel')
        self.verticalComboBox.setObjectName(u'verticalComboBox')
        self.fontLayout.addRow(self.verticalLabel, self.verticalComboBox)
        self.leftLayout.addWidget(self.fontGroupBox)
        self.leftLayout.addStretch()
        self.previewGroupBox = QtGui.QGroupBox(self.rightColumn)
        self.previewGroupBox.setObjectName(u'previewGroupBox')
        self.previewLayout = QtGui.QVBoxLayout(self.previewGroupBox)
        self.previewLayout.setObjectName(u'previewLayout')
        self.fontPreview = QtGui.QLineEdit(self.previewGroupBox)
        self.fontPreview.setObjectName(u'fontPreview')
        self.previewLayout.addWidget(self.fontPreview)
        self.rightLayout.addWidget(self.previewGroupBox)
        self.rightLayout.addStretch()
        # Signals and slots
        QtCore.QObject.connect(self.backgroundColorButton,
            QtCore.SIGNAL(u'clicked()'), self.onBackgroundColorButtonClicked)
        QtCore.QObject.connect(self.fontColorButton,
            QtCore.SIGNAL(u'clicked()'), self.onFontColorButtonClicked)
        QtCore.QObject.connect(self.fontComboBox,
            QtCore.SIGNAL(u'activated(int)'), self.onFontComboBoxClicked)
        QtCore.QObject.connect(self.timeoutSpinBox,
            QtCore.SIGNAL(u'valueChanged(int)'), self.onTimeoutSpinBoxChanged)
        QtCore.QObject.connect(self.fontSizeSpinBox,
            QtCore.SIGNAL(u'valueChanged(int)'), self.onFontSizeSpinBoxChanged)

    def retranslateUi(self):
        self.fontGroupBox.setTitle(
            translate('AlertsPlugin.AlertsTab', 'Font'))
        self.fontLabel.setText(
            translate('AlertsPlugin.AlertsTab', 'Font name:'))
        self.fontColorLabel.setText(
            translate('AlertsPlugin.AlertsTab', 'Font color:'))
        self.backgroundColorLabel.setText(
            translate('AlertsPlugin.AlertsTab', 'Background color:'))
        self.fontSizeLabel.setText(
            translate('AlertsPlugin.AlertsTab', 'Font size:'))
        self.fontSizeSpinBox.setSuffix(UiStrings().FontSizePtUnit)
        self.timeoutLabel.setText(
            translate('AlertsPlugin.AlertsTab', 'Alert timeout:'))
        self.timeoutSpinBox.setSuffix(UiStrings().Seconds)
        self.previewGroupBox.setTitle(UiStrings().Preview)
        self.fontPreview.setText(UiStrings().OLPV2)

    def onBackgroundColorButtonClicked(self):
        new_color = QtGui.QColorDialog.getColor(
            QtGui.QColor(self.bg_color), self)
        if new_color.isValid():
            self.bg_color = new_color.name()
            self.backgroundColorButton.setStyleSheet(
                u'background-color: %s' % self.bg_color)
            self.updateDisplay()

    def onFontComboBoxClicked(self):
        self.updateDisplay()

    def onFontColorButtonClicked(self):
        new_color = QtGui.QColorDialog.getColor(
            QtGui.QColor(self.font_color), self)
        if new_color.isValid():
            self.font_color = new_color.name()
            self.fontColorButton.setStyleSheet(
                u'background-color: %s' % self.font_color)
            self.updateDisplay()

    def onTimeoutSpinBoxChanged(self):
        self.timeout = self.timeoutSpinBox.value()
        self.changed = True

    def onFontSizeSpinBoxChanged(self):
        self.font_size = self.fontSizeSpinBox.value()
        self.updateDisplay()

    def load(self):
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        self.timeout = settings.value(u'timeout', QtCore.QVariant(5)).toInt()[0]
        self.font_color = unicode(settings.value(
            u'font color', QtCore.QVariant(u'#ffffff')).toString())
        self.font_size = settings.value(
            u'font size', QtCore.QVariant(40)).toInt()[0]
        self.bg_color = unicode(settings.value(
            u'background color', QtCore.QVariant(u'#660000')).toString())
        self.font_face = unicode(settings.value(
            u'font face', QtCore.QVariant(QtGui.QFont().family())).toString())
        self.location = settings.value(
            u'location', QtCore.QVariant(AlertLocation.Bottom)).toInt()[0]
        settings.endGroup()
        self.fontSizeSpinBox.setValue(self.font_size)
        self.timeoutSpinBox.setValue(self.timeout)
        self.fontColorButton.setStyleSheet(
            u'background-color: %s' % self.font_color)
        self.backgroundColorButton.setStyleSheet(
            u'background-color: %s' % self.bg_color)
        self.verticalComboBox.setCurrentIndex(self.location)
        font = QtGui.QFont()
        font.setFamily(self.font_face)
        self.fontComboBox.setCurrentFont(font)
        self.updateDisplay()
        self.changed = False

    def save(self):
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        # Check value has changed as no event handles this field
        if settings.value(u'location', QtCore.QVariant(1)).toInt()[0] != \
            self.verticalComboBox.currentIndex():
            self.changed = True
        settings.setValue(u'background color', QtCore.QVariant(self.bg_color))
        settings.setValue(u'font color', QtCore.QVariant(self.font_color))
        settings.setValue(u'font size', QtCore.QVariant(self.font_size))
        self.font_face = self.fontComboBox.currentFont().family()
        settings.setValue(u'font face', QtCore.QVariant(self.font_face))
        settings.setValue(u'timeout', QtCore.QVariant(self.timeout))
        self.location = self.verticalComboBox.currentIndex()
        settings.setValue(u'location', QtCore.QVariant(self.location))
        settings.endGroup()
        if self.changed:
            Receiver.send_message(u'update_display_css')
        self.changed = False

    def updateDisplay(self):
        font = QtGui.QFont()
        font.setFamily(self.fontComboBox.currentFont().family())
        font.setBold(True)
        font.setPointSize(self.font_size)
        self.fontPreview.setFont(font)
        self.fontPreview.setStyleSheet(u'background-color: %s; color: %s' %
            (self.bg_color, self.font_color))
        self.changed = True




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The First Time Wizard
"""
import io
import logging
import os
import sys
import time
import urllib
import urllib2
from tempfile import gettempdir
from ConfigParser import SafeConfigParser

from PyQt4 import QtCore, QtGui

from openlp.core.lib import translate, PluginStatus, Receiver, build_icon, \
    check_directory_exists
from openlp.core.lib.settings import Settings
from openlp.core.utils import get_web_page, AppLocation, join_url, \
    get_filesystem_encoding, get_application_version
from firsttimewizard import Ui_FirstTimeWizard, FirstTimePage

log = logging.getLogger(__name__)


class ThemeScreenshotThread(QtCore.QThread):
    """
    This thread downloads the theme screenshots.
    """
    def __init__(self, parent):
        QtCore.QThread.__init__(self, parent)

    def run(self):
        """
        Run the thread.
        """
        themes = self.parent().config.get(u'themes', u'files')
        themes = themes.split(u',')
        themes_dir = self.parent().config.get(u'themes', u'directory')
        tmp_dir = os.path.join(unicode(gettempdir(), get_filesystem_encoding()),
                u'openlp')
        config = self.parent().config
        for theme in themes:
            # Stop if the wizard has been cancelled.
            if self.parent().downloadCancelled:
                return
            title = config.get(u'theme_%s' % theme, u'title')
            filename = config.get(u'theme_%s' % theme, u'filename')
            screenshot = config.get(u'theme_%s' % theme, u'screenshot')
            urllib.urlretrieve(join_url(self.parent().baseurl,
                themes_dir, screenshot),
                os.path.join(tmp_dir, screenshot))
            item = QtGui.QListWidgetItem(title, self.parent().themesListWidget)
            item.setData(QtCore.Qt.UserRole, QtCore.QVariant(filename))
            item.setCheckState(QtCore.Qt.Unchecked)
            item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)


class FirstTimeForm(QtGui.QWizard, Ui_FirstTimeWizard):
    """
    This is the Theme Import Wizard, which allows easy creation and editing of
    OpenLP themes.
    """
    log.info(u'ThemeWizardForm loaded')

    def __init__(self, screens, parent=None):
        QtGui.QWizard.__init__(self, parent)
        self.setupUi(self)
        self.screens = screens
        self.config = SafeConfigParser()
        # The following url should contain 'download.cfg' or redirect OpenLP
        # to the right url of 'download.cfg' file.
        # 'download.cfg' file contains definitions for everything that is
        # available for download into OpenLP in FirstTimeWizard.
        self.web = u'http://openlp.org/files/frw/'
        # Base url for dowloading resource files (songs, themes, bibles).
        # These files could be hosted on different places and thus the base
        # url is defined in 'download.cfg' file.
        self.baseurl = None
        # Check to see if we have web access
        user_agent = u'OpenLP/' + get_application_version()[u'version']
        self.webAccess = get_web_page(u'%s%s' % (self.web, u'download.cfg'), header=(u'User-Agent', user_agent))
        if self.webAccess:
            files = self.webAccess.read()
            self.config.readfp(io.BytesIO(files))
            self.baseurl = self.config.get(u'general', u'base url')
        self.updateScreenListCombo()
        self.downloadCancelled = False
        self.downloading = unicode(translate('OpenLP.FirstTimeWizard',
            'Downloading %s...'))
        QtCore.QObject.connect(self.cancelButton, QtCore.SIGNAL('clicked()'),
            self.onCancelButtonClicked)
        QtCore.QObject.connect(self.noInternetFinishButton,
            QtCore.SIGNAL('clicked()'), self.onNoInternetFinishButtonClicked)
        QtCore.QObject.connect(self,
            QtCore.SIGNAL(u'currentIdChanged(int)'), self.onCurrentIdChanged)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'config_screen_changed'), self.updateScreenListCombo)

    def exec_(self, edit=False):
        """
        Run the wizard.
        """
        self.setDefaults()
        return QtGui.QWizard.exec_(self)

    def setDefaults(self):
        """
        Set up display at start of theme edit.
        """
        self.restart()
        check_directory_exists(os.path.join(
            unicode(gettempdir(), get_filesystem_encoding()), u'openlp'))
        self.noInternetFinishButton.setVisible(False)
        # Check if this is a re-run of the wizard.
        self.hasRunWizard = Settings().value(
            u'general/has run wizard', QtCore.QVariant(False)).toBool()
        # Sort out internet access for downloads.
        if self.webAccess:
            songs = self.config.get(u'songs', u'languages')
            songs = songs.split(u',')
            for song in songs:
                title = unicode(self.config.get(
                    u'songs_%s' % song, u'title'), u'utf8')
                filename = unicode(self.config.get(
                    u'songs_%s' % song, u'filename'), u'utf8')
                item = QtGui.QListWidgetItem(title, self.songsListWidget)
                item.setData(QtCore.Qt.UserRole, QtCore.QVariant(filename))
                item.setCheckState(QtCore.Qt.Unchecked)
                item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)
            bible_languages = self.config.get(u'bibles', u'languages')
            bible_languages = bible_languages.split(u',')
            for lang in bible_languages:
                language = unicode(self.config.get(
                    u'bibles_%s' % lang, u'title'), u'utf8')
                langItem = QtGui.QTreeWidgetItem(
                    self.biblesTreeWidget, QtCore.QStringList(language))
                bibles = self.config.get(u'bibles_%s' % lang, u'translations')
                bibles = bibles.split(u',')
                for bible in bibles:
                    title = unicode(self.config.get(
                        u'bible_%s' % bible, u'title'), u'utf8')
                    filename = unicode(self.config.get(
                        u'bible_%s' % bible, u'filename'))
                    item = QtGui.QTreeWidgetItem(
                        langItem, QtCore.QStringList(title))
                    item.setData(0, QtCore.Qt.UserRole,
                        QtCore.QVariant(filename))
                    item.setCheckState(0, QtCore.Qt.Unchecked)
                    item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)
            self.biblesTreeWidget.expandAll()
            # Download the theme screenshots.
            self.themeScreenshotThread = ThemeScreenshotThread(self)
            self.themeScreenshotThread.start()
        Receiver.send_message(u'cursor_normal')

    def nextId(self):
        """
        Determine the next page in the Wizard to go to.
        """
        Receiver.send_message(u'openlp_process_events')
        # If we are currently on the plugins page
        if self.currentId() == FirstTimePage.Plugins:
            # But we don't have Internet access
            if not self.webAccess:
                return FirstTimePage.NoInternet
            # The songs plugin is enabled
            elif self.songsCheckBox.isChecked():
                return FirstTimePage.Songs
            # The Bibles plugin is enabled
            elif self.bibleCheckBox.isChecked():
                return FirstTimePage.Bibles
            else:
                return FirstTimePage.Themes
        elif self.currentId() == FirstTimePage.Progress:
            return -1
        elif self.currentId() == FirstTimePage.NoInternet:
            return FirstTimePage.Progress
        elif self.currentId() == FirstTimePage.Songs:
            if self.bibleCheckBox.isChecked():
                return FirstTimePage.Bibles
            else:
                return FirstTimePage.Themes
        elif self.currentId() == FirstTimePage.Themes:
            Receiver.send_message(u'cursor_busy')
            Receiver.send_message(u'openlp_process_events')
            while not self.themeScreenshotThread.isFinished():
                time.sleep(0.1)
                Receiver.send_message(u'openlp_process_events')
            # Build the screenshot icons, as this can not be done in the thread.
            self._buildThemeScreenshots()
            Receiver.send_message(u'cursor_normal')
            return FirstTimePage.Defaults
        else:
            return self.currentId() + 1

    def onCurrentIdChanged(self, pageId):
        """
        Detects Page changes and updates as appropriate.
        """
        # Keep track of the page we are at.  Triggering "Cancel" causes pageId
        # to be a -1.
        Receiver.send_message(u'openlp_process_events')
        if pageId != -1:
            self.lastId = pageId
        if pageId == FirstTimePage.Plugins:
            # Set the no internet page text.
            if self.hasRunWizard:
                self.noInternetLabel.setText(self.noInternetText)
            else:
                self.noInternetLabel.setText(self.noInternetText +
                    self.cancelWizardText)
        elif pageId == FirstTimePage.Defaults:
            self.themeComboBox.clear()
            for iter in xrange(self.themesListWidget.count()):
                item = self.themesListWidget.item(iter)
                if item.checkState() == QtCore.Qt.Checked:
                    self.themeComboBox.addItem(item.text())
            if self.hasRunWizard:
                # Add any existing themes to list.
                for theme in self.parent().themeManagerContents.getThemes():
                    index = self.themeComboBox.findText(theme)
                    if index == -1:
                        self.themeComboBox.addItem(theme)
                default_theme = unicode(Settings().value(
                    u'themes/global theme',
                    QtCore.QVariant(u'')).toString())
                # Pre-select the current default theme.
                index = self.themeComboBox.findText(default_theme)
                self.themeComboBox.setCurrentIndex(index)
        elif pageId == FirstTimePage.NoInternet:
            self.backButton.setVisible(False)
            self.nextButton.setVisible(False)
            self.noInternetFinishButton.setVisible(True)
            if self.hasRunWizard:
                self.cancelButton.setVisible(False)
        elif pageId == FirstTimePage.Progress:
            Receiver.send_message(u'cursor_busy')
            self.repaint()
            Receiver.send_message(u'openlp_process_events')
            # Try to give the wizard a chance to redraw itself
            time.sleep(0.2)
            self._preWizard()
            self._performWizard()
            self._postWizard()
            Receiver.send_message(u'cursor_normal')
            Receiver.send_message(u'openlp_process_events')

    def updateScreenListCombo(self):
        """
        The user changed screen resolution or enabled/disabled more screens, so
        we need to update the combo box.
        """
        self.displayComboBox.clear()
        self.displayComboBox.addItems(self.screens.get_screen_list())
        self.displayComboBox.setCurrentIndex(self.displayComboBox.count() - 1)

    def onCancelButtonClicked(self):
        """
        Process the triggering of the cancel button.
        """
        if self.lastId == FirstTimePage.NoInternet or \
            (self.lastId <= FirstTimePage.Plugins and not self.hasRunWizard):
            QtCore.QCoreApplication.exit()
            sys.exit()
        self.downloadCancelled = True
        while self.themeScreenshotThread.isRunning():
            time.sleep(0.1)
        Receiver.send_message(u'cursor_normal')

    def onNoInternetFinishButtonClicked(self):
        """
        Process the triggering of the "Finish" button on the No Internet page.
        """
        Receiver.send_message(u'cursor_busy')
        self._performWizard()
        Receiver.send_message(u'cursor_normal')
        Receiver.send_message(u'openlp_process_events')
        Settings().setValue(u'general/has run wizard',
            QtCore.QVariant(True))
        self.close()

    def urlGetFile(self, url, fpath):
        """"
        Download a file given a URL.  The file is retrieved in chunks, giving
        the ability to cancel the download at any point.
        """
        log.debug(u'Downloading %s' % url)
        block_count = 0
        block_size = 4096
        urlfile = urllib2.urlopen(url)
        filesize = urlfile.headers["Content-Length"]
        filename = open(fpath, "wb")
        # Download until finished or canceled.
        while not self.downloadCancelled:
            data = urlfile.read(block_size)
            if not data:
                break
            filename.write(data)
            block_count += 1
            self._downloadProgress(block_count, block_size, filesize)
        filename.close()
        # Delete file if cancelled, it may be a partial file.
        if self.downloadCancelled:
            os.remove(fpath)

    def _buildThemeScreenshots(self):
        """
        This method builds the theme screenshots' icons for all items in the
        ``self.themesListWidget``.
        """
        themes = self.config.get(u'themes', u'files')
        themes = themes.split(u',')
        for theme in themes:
            filename = self.config.get(u'theme_%s' % theme, u'filename')
            screenshot = self.config.get(u'theme_%s' % theme, u'screenshot')
            for index in xrange(self.themesListWidget.count()):
                item = self.themesListWidget.item(index)
                if item.data(QtCore.Qt.UserRole) == QtCore.QVariant(filename):
                    break
            item.setIcon(build_icon(os.path.join(unicode(gettempdir(),
                get_filesystem_encoding()), u'openlp', screenshot)))

    def _getFileSize(self, url):
        site = urllib.urlopen(url)
        meta = site.info()
        return int(meta.getheaders("Content-Length")[0])

    def _downloadProgress(self, count, block_size, total_size):
        increment = (count * block_size) - self.previous_size
        self._incrementProgressBar(None, increment)
        self.previous_size = count * block_size

    def _incrementProgressBar(self, status_text, increment=1):
        """
        Update the wizard progress page.

        ``status_text``
            Current status information to display.

        ``increment``
            The value to increment the progress bar by.
        """
        if status_text:
            self.progressLabel.setText(status_text)
        if increment > 0:
            self.progressBar.setValue(self.progressBar.value() + increment)
        Receiver.send_message(u'openlp_process_events')

    def _preWizard(self):
        """
        Prepare the UI for the process.
        """
        self.max_progress = 0
        self.finishButton.setVisible(False)
        Receiver.send_message(u'openlp_process_events')
        # Directory name appended to base download url as the name
        # is found in download.cfg file.
        songs_dir = unicode(self.config.get(u'songs', u'directory'), u'utf8')
        bibles_dir = unicode(self.config.get(u'bibles', u'directory'), u'utf8')
        themes_dir = unicode(self.config.get(u'themes', u'directory'), u'utf8')
        # Loop through the songs list and increase for each selected item
        for i in xrange(self.songsListWidget.count()):
            Receiver.send_message(u'openlp_process_events')
            item = self.songsListWidget.item(i)
            if item.checkState() == QtCore.Qt.Checked:
                filename = item.data(QtCore.Qt.UserRole).toString()
                size = self._getFileSize(join_url(self.baseurl, songs_dir, filename))
                self.max_progress += size
        # Loop through the Bibles list and increase for each selected item
        iterator = QtGui.QTreeWidgetItemIterator(self.biblesTreeWidget)
        while iterator.value():
            Receiver.send_message(u'openlp_process_events')
            item = iterator.value()
            if item.parent() and item.checkState(0) == QtCore.Qt.Checked:
                filename = item.data(0, QtCore.Qt.UserRole).toString()
                size = self._getFileSize(join_url(self.baseurl, bibles_dir, filename))
                self.max_progress += size
            iterator += 1
        # Loop through the themes list and increase for each selected item
        for i in xrange(self.themesListWidget.count()):
            Receiver.send_message(u'openlp_process_events')
            item = self.themesListWidget.item(i)
            if item.checkState() == QtCore.Qt.Checked:
                filename = item.data(QtCore.Qt.UserRole).toString()
                size = self._getFileSize(join_url(self.baseurl, themes_dir, filename))
                self.max_progress += size
        if self.max_progress:
            # Add on 2 for plugins status setting plus a "finished" point.
            self.max_progress += 2
            self.progressBar.setValue(0)
            self.progressBar.setMinimum(0)
            self.progressBar.setMaximum(self.max_progress)
            self.progressPage.setTitle(translate('OpenLP.FirstTimeWizard',
                'Setting Up And Downloading'))
            self.progressPage.setSubTitle(translate('OpenLP.FirstTimeWizard',
                'Please wait while OpenLP is set up '
                'and your data is downloaded.'))
        else:
            self.progressBar.setVisible(False)
            self.progressPage.setTitle(translate('OpenLP.FirstTimeWizard',
                'Setting Up'))
            self.progressPage.setSubTitle(u'Setup complete.')
        self.repaint()
        Receiver.send_message(u'openlp_process_events')
        # Try to give the wizard a chance to repaint itself
        time.sleep(0.1)

    def _postWizard(self):
        """
        Clean up the UI after the process has finished.
        """
        if self.max_progress:
            self.progressBar.setValue(self.progressBar.maximum())
            if self.hasRunWizard:
                self.progressLabel.setText(translate('OpenLP.FirstTimeWizard',
                    'Download complete.'
                    ' Click the finish button to return to OpenLP.'))
            else:
                self.progressLabel.setText(translate('OpenLP.FirstTimeWizard',
                    'Download complete.'
                    ' Click the finish button to start OpenLP.'))
        else:
            if self.hasRunWizard:
                self.progressLabel.setText(translate('OpenLP.FirstTimeWizard',
                    'Click the finish button to return to OpenLP.'))
            else:
                self.progressLabel.setText(translate('OpenLP.FirstTimeWizard',
                    'Click the finish button to start OpenLP.'))
        self.finishButton.setVisible(True)
        self.finishButton.setEnabled(True)
        self.cancelButton.setVisible(False)
        self.nextButton.setVisible(False)
        Receiver.send_message(u'openlp_process_events')

    def _performWizard(self):
        """
        Run the tasks in the wizard.
        """
        # Set plugin states
        self._incrementProgressBar(translate('OpenLP.FirstTimeWizard',
            'Enabling selected plugins...'))
        self._setPluginStatus(self.songsCheckBox, u'songs/status')
        self._setPluginStatus(self.bibleCheckBox, u'bibles/status')
        # TODO Presentation plugin is not yet working on Mac OS X.
        # For now just ignore it.
        if sys.platform != 'darwin':
            self._setPluginStatus(self.presentationCheckBox,
                u'presentations/status')
        self._setPluginStatus(self.imageCheckBox, u'images/status')
        self._setPluginStatus(self.mediaCheckBox, u'media/status')
        self._setPluginStatus(self.remoteCheckBox, u'remotes/status')
        self._setPluginStatus(self.customCheckBox, u'custom/status')
        self._setPluginStatus(self.songUsageCheckBox, u'songusage/status')
        self._setPluginStatus(self.alertCheckBox, u'alerts/status')
        if self.webAccess:
            # Build directories for downloads
            songs_destination = os.path.join(
                unicode(gettempdir(), get_filesystem_encoding()), u'openlp')
            bibles_destination = AppLocation.get_section_data_path(u'bibles')
            themes_destination = AppLocation.get_section_data_path(u'themes')
            # Directory name appended to base download url as the name
            # is found in download.cfg file.
            songs_dir = unicode(self.config.get(u'songs', u'directory'), u'utf8')
            bibles_dir = unicode(self.config.get(u'bibles', u'directory'), u'utf8')
            themes_dir = unicode(self.config.get(u'themes', u'directory'), u'utf8')
            # Download songs
            for i in xrange(self.songsListWidget.count()):
                item = self.songsListWidget.item(i)
                if item.checkState() == QtCore.Qt.Checked:
                    filename = item.data(QtCore.Qt.UserRole).toString()
                    self._incrementProgressBar(self.downloading % filename, 0)
                    self.previous_size = 0
                    destination = os.path.join(songs_destination,
                        unicode(filename))
                    self.urlGetFile(join_url(self.baseurl, songs_dir, filename),
                        destination)
            # Download Bibles
            bibles_iterator = QtGui.QTreeWidgetItemIterator(
                self.biblesTreeWidget)
            while bibles_iterator.value():
                item = bibles_iterator.value()
                if item.parent() and item.checkState(0) == QtCore.Qt.Checked:
                    bible = unicode(item.data(0, QtCore.Qt.UserRole).toString())
                    self._incrementProgressBar(self.downloading % bible, 0)
                    self.previous_size = 0
                    self.urlGetFile(join_url(self.baseurl, bibles_dir, bible),
                        os.path.join(bibles_destination, bible))
                bibles_iterator += 1
            # Download themes
            for i in xrange(self.themesListWidget.count()):
                item = self.themesListWidget.item(i)
                if item.checkState() == QtCore.Qt.Checked:
                    theme = unicode(item.data(QtCore.Qt.UserRole).toString())
                    self._incrementProgressBar(self.downloading % theme, 0)
                    self.previous_size = 0
                    self.urlGetFile(join_url(self.baseurl, themes_dir, theme),
                        os.path.join(themes_destination, theme))
        # Set Default Display
        if self.displayComboBox.currentIndex() != -1:
            Settings().setValue(u'General/monitor',
                QtCore.QVariant(self.displayComboBox.currentIndex()))
            self.screens.set_current_display(
                 self.displayComboBox.currentIndex())
        # Set Global Theme
        if self.themeComboBox.currentIndex() != -1:
            Settings().setValue(u'themes/global theme',
                QtCore.QVariant(self.themeComboBox.currentText()))

    def _setPluginStatus(self, field, tag):
        status = PluginStatus.Active if field.checkState() \
            == QtCore.Qt.Checked else PluginStatus.Inactive
        Settings().setValue(tag, QtCore.QVariant(status))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

import sys

from openlp.core.lib import translate, build_icon
from openlp.core.lib.ui import add_welcome_page


class FirstTimePage(object):
    """
    An enumeration object to make it easy for a developer to determine which page is which by index
    """
    Welcome = 0
    Plugins = 1
    NoInternet = 2
    Songs = 3
    Bibles = 4
    Themes = 5
    Defaults = 6
    Progress = 7


class Ui_FirstTimeWizard(object):
    def setupUi(self, FirstTimeWizard):
        FirstTimeWizard.setObjectName(u'FirstTimeWizard')
        FirstTimeWizard.resize(550, 386)
        FirstTimeWizard.setModal(True)
        FirstTimeWizard.setWizardStyle(QtGui.QWizard.ModernStyle)
        FirstTimeWizard.setOptions(QtGui.QWizard.IndependentPages |
            QtGui.QWizard.NoBackButtonOnStartPage |
            QtGui.QWizard.NoBackButtonOnLastPage |
            QtGui.QWizard.HaveCustomButton1)
        FirstTimeWizard.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.finishButton = self.button(QtGui.QWizard.FinishButton)
        self.noInternetFinishButton = self.button(QtGui.QWizard.CustomButton1)
        self.cancelButton = self.button(QtGui.QWizard.CancelButton)
        self.nextButton = self.button(QtGui.QWizard.NextButton)
        self.backButton = self.button(QtGui.QWizard.BackButton)
        add_welcome_page(FirstTimeWizard, u':/wizards/wizard_firsttime.bmp')
        # The plugins page
        self.pluginPage = QtGui.QWizardPage()
        self.pluginPage.setObjectName(u'pluginPage')
        self.pluginLayout = QtGui.QVBoxLayout(self.pluginPage)
        self.pluginLayout.setContentsMargins(40, 15, 40, 0)
        self.pluginLayout.setObjectName(u'pluginLayout')
        self.songsCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.songsCheckBox.setChecked(True)
        self.songsCheckBox.setObjectName(u'songsCheckBox')
        self.pluginLayout.addWidget(self.songsCheckBox)
        self.customCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.customCheckBox.setChecked(True)
        self.customCheckBox.setObjectName(u'customCheckBox')
        self.pluginLayout.addWidget(self.customCheckBox)
        self.bibleCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.bibleCheckBox.setChecked(True)
        self.bibleCheckBox.setObjectName(u'bibleCheckBox')
        self.pluginLayout.addWidget(self.bibleCheckBox)
        self.imageCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.imageCheckBox.setChecked(True)
        self.imageCheckBox.setObjectName(u'imageCheckBox')
        self.pluginLayout.addWidget(self.imageCheckBox)
        # TODO Presentation plugin is not yet working on Mac OS X.
        # For now just ignore it.
        if sys.platform != 'darwin':
            self.presentationCheckBox = QtGui.QCheckBox(self.pluginPage)
            self.presentationCheckBox.setChecked(True)
            self.presentationCheckBox.setObjectName(u'presentationCheckBox')
            self.pluginLayout.addWidget(self.presentationCheckBox)
        self.mediaCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.mediaCheckBox.setChecked(True)
        self.mediaCheckBox.setObjectName(u'mediaCheckBox')
        self.pluginLayout.addWidget(self.mediaCheckBox)
        self.remoteCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.remoteCheckBox.setObjectName(u'remoteCheckBox')
        self.pluginLayout.addWidget(self.remoteCheckBox)
        self.songUsageCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.songUsageCheckBox.setChecked(True)
        self.songUsageCheckBox.setObjectName(u'songUsageCheckBox')
        self.pluginLayout.addWidget(self.songUsageCheckBox)
        self.alertCheckBox = QtGui.QCheckBox(self.pluginPage)
        self.alertCheckBox.setChecked(True)
        self.alertCheckBox.setObjectName(u'alertCheckBox')
        self.pluginLayout.addWidget(self.alertCheckBox)
        FirstTimeWizard.setPage(FirstTimePage.Plugins, self.pluginPage)
        # The "you don't have an internet connection" page.
        self.noInternetPage = QtGui.QWizardPage()
        self.noInternetPage.setObjectName(u'noInternetPage')
        self.noInternetLayout = QtGui.QVBoxLayout(self.noInternetPage)
        self.noInternetLayout.setContentsMargins(50, 30, 50, 40)
        self.noInternetLayout.setObjectName(u'noInternetLayout')
        self.noInternetLabel = QtGui.QLabel(self.noInternetPage)
        self.noInternetLabel.setWordWrap(True)
        self.noInternetLabel.setObjectName(u'noInternetLabel')
        self.noInternetLayout.addWidget(self.noInternetLabel)
        FirstTimeWizard.setPage(FirstTimePage.NoInternet, self.noInternetPage)
        # The song samples page
        self.songsPage = QtGui.QWizardPage()
        self.songsPage.setObjectName(u'songsPage')
        self.songsLayout = QtGui.QVBoxLayout(self.songsPage)
        self.songsLayout.setContentsMargins(50, 20, 50, 20)
        self.songsLayout.setObjectName(u'songsLayout')
        self.songsListWidget = QtGui.QListWidget(self.songsPage)
        self.songsListWidget.setAlternatingRowColors(True)
        self.songsListWidget.setObjectName(u'songsListWidget')
        self.songsLayout.addWidget(self.songsListWidget)
        FirstTimeWizard.setPage(FirstTimePage.Songs, self.songsPage)
        # The Bible samples page
        self.biblesPage = QtGui.QWizardPage()
        self.biblesPage.setObjectName(u'biblesPage')
        self.biblesLayout = QtGui.QVBoxLayout(self.biblesPage)
        self.biblesLayout.setContentsMargins(50, 20, 50, 20)
        self.biblesLayout.setObjectName(u'biblesLayout')
        self.biblesTreeWidget = QtGui.QTreeWidget(self.biblesPage)
        self.biblesTreeWidget.setAlternatingRowColors(True)
        self.biblesTreeWidget.header().setVisible(False)
        self.biblesTreeWidget.setObjectName(u'biblesTreeWidget')
        self.biblesLayout.addWidget(self.biblesTreeWidget)
        FirstTimeWizard.setPage(FirstTimePage.Bibles, self.biblesPage)
        # The theme samples page
        self.themesPage = QtGui.QWizardPage()
        self.themesPage.setObjectName(u'themesPage')
        self.themesLayout = QtGui.QVBoxLayout(self.themesPage)
        self.themesLayout.setContentsMargins(20, 50, 20, 60)
        self.themesLayout.setObjectName(u'themesLayout')
        self.themesListWidget = QtGui.QListWidget(self.themesPage)
        self.themesListWidget.setViewMode(QtGui.QListView.IconMode)
        self.themesListWidget.setMovement(QtGui.QListView.Static)
        self.themesListWidget.setFlow(QtGui.QListView.LeftToRight)
        self.themesListWidget.setSpacing(4)
        self.themesListWidget.setUniformItemSizes(True)
        self.themesListWidget.setIconSize(QtCore.QSize(133, 100))
        self.themesListWidget.setWrapping(False)
        self.themesListWidget.setObjectName(u'themesListWidget')
        self.themesLayout.addWidget(self.themesListWidget)
        FirstTimeWizard.setPage(FirstTimePage.Themes, self.themesPage)
        # the default settings page
        self.defaultsPage = QtGui.QWizardPage()
        self.defaultsPage.setObjectName(u'defaultsPage')
        self.defaultsLayout = QtGui.QFormLayout(self.defaultsPage)
        self.defaultsLayout.setContentsMargins(50, 20, 50, 20)
        self.defaultsLayout.setObjectName(u'defaultsLayout')
        self.displayLabel = QtGui.QLabel(self.defaultsPage)
        self.displayLabel.setObjectName(u'displayLabel')
        self.displayComboBox = QtGui.QComboBox(self.defaultsPage)
        self.displayComboBox.setEditable(False)
        self.displayComboBox.setInsertPolicy(QtGui.QComboBox.NoInsert)
        self.displayComboBox.setObjectName(u'displayComboBox')
        self.defaultsLayout.addRow(self.displayLabel, self.displayComboBox)
        self.themeLabel = QtGui.QLabel(self.defaultsPage)
        self.themeLabel.setObjectName(u'themeLabel')
        self.themeComboBox = QtGui.QComboBox(self.defaultsPage)
        self.themeComboBox.setEditable(False)
        self.themeComboBox.setInsertPolicy(QtGui.QComboBox.NoInsert)
        self.themeComboBox.setSizeAdjustPolicy(
            QtGui.QComboBox.AdjustToContents)
        self.themeComboBox.setObjectName(u'themeComboBox')
        self.defaultsLayout.addRow(self.themeLabel, self.themeComboBox)
        FirstTimeWizard.setPage(FirstTimePage.Defaults, self.defaultsPage)
        # Progress page
        self.progressPage = QtGui.QWizardPage()
        self.progressPage.setObjectName(u'progressPage')
        self.progressLayout = QtGui.QVBoxLayout(self.progressPage)
        self.progressLayout.setMargin(48)
        self.progressLayout.setObjectName(u'progressLayout')
        self.progressLabel = QtGui.QLabel(self.progressPage)
        self.progressLabel.setObjectName(u'progressLabel')
        self.progressLayout.addWidget(self.progressLabel)
        self.progressBar = QtGui.QProgressBar(self.progressPage)
        self.progressBar.setObjectName(u'progressBar')
        self.progressLayout.addWidget(self.progressBar)
        FirstTimeWizard.setPage(FirstTimePage.Progress, self.progressPage)
        self.retranslateUi(FirstTimeWizard)

    def retranslateUi(self, FirstTimeWizard):
        FirstTimeWizard.setWindowTitle(translate(
            'OpenLP.FirstTimeWizard', 'First Time Wizard'))
        self.titleLabel.setText(
            u'<span style="font-size:14pt; font-weight:600;">%s</span>' % \
            translate('OpenLP.FirstTimeWizard',
            'Welcome to the First Time Wizard'))
        self.informationLabel.setText(translate('OpenLP.FirstTimeWizard',
            'This wizard will help you to configure OpenLP for initial use.'
            ' Click the next button below to start.'))
        self.pluginPage.setTitle(translate('OpenLP.FirstTimeWizard',
            'Activate required Plugins'))
        self.pluginPage.setSubTitle(translate('OpenLP.FirstTimeWizard',
            'Select the Plugins you wish to use. '))
        self.songsCheckBox.setText(translate('OpenLP.FirstTimeWizard', 'Songs'))
        self.customCheckBox.setText(translate('OpenLP.FirstTimeWizard',
            'Custom Slides'))
        self.bibleCheckBox.setText(translate('OpenLP.FirstTimeWizard', 'Bible'))
        self.imageCheckBox.setText(translate('OpenLP.FirstTimeWizard',
            'Images'))
        # TODO Presentation plugin is not yet working on Mac OS X.
        # For now just ignore it.
        if sys.platform != 'darwin':
            self.presentationCheckBox.setText(translate('OpenLP.FirstTimeWizard',
                'Presentations'))
        self.mediaCheckBox.setText(translate('OpenLP.FirstTimeWizard',
            'Media (Audio and Video)'))
        self.remoteCheckBox.setText(translate('OpenLP.FirstTimeWizard',
            'Allow remote access'))
        self.songUsageCheckBox.setText(translate('OpenLP.FirstTimeWizard',
            'Monitor Song Usage'))
        self.alertCheckBox.setText(translate('OpenLP.FirstTimeWizard',
            'Allow Alerts'))
        self.noInternetPage.setTitle(translate('OpenLP.FirstTimeWizard',
            'No Internet Connection'))
        self.noInternetPage.setSubTitle(translate(
            'OpenLP.FirstTimeWizard',
            'Unable to detect an Internet connection.'))
        self.noInternetText = translate('OpenLP.FirstTimeWizard',
            'No Internet connection was found. The First Time Wizard needs an '
            'Internet connection in order to be able to download sample '
            'songs, Bibles and themes.  Click the Finish button now to start '
            'OpenLP with initial settings and no sample data.\n\nTo re-run the '
            'First Time Wizard and import this sample data at a later time, '
            'check your Internet connection and re-run this wizard by '
            'selecting "Tools/Re-run First Time Wizard" from OpenLP.')
        self.cancelWizardText = translate('OpenLP.FirstTimeWizard',
            '\n\nTo cancel the First Time Wizard completely (and not start '
            'OpenLP), click the Cancel button now.')
        self.songsPage.setTitle(translate('OpenLP.FirstTimeWizard',
            'Sample Songs'))
        self.songsPage.setSubTitle(translate('OpenLP.FirstTimeWizard',
            'Select and download public domain songs.'))
        self.biblesPage.setTitle(translate('OpenLP.FirstTimeWizard',
            'Sample Bibles'))
        self.biblesPage.setSubTitle(translate('OpenLP.FirstTimeWizard',
            'Select and download free Bibles.'))
        self.themesPage.setTitle(translate('OpenLP.FirstTimeWizard',
            'Sample Themes'))
        self.themesPage.setSubTitle(translate('OpenLP.FirstTimeWizard',
            'Select and download sample themes.'))
        self.defaultsPage.setTitle(translate('OpenLP.FirstTimeWizard',
            'Default Settings'))
        self.defaultsPage.setSubTitle(translate('OpenLP.FirstTimeWizard',
            'Set up default settings to be used by OpenLP.'))
        self.displayLabel.setText(translate('OpenLP.FirstTimeWizard',
            'Default output display:'))
        self.themeLabel.setText(translate('OpenLP.FirstTimeWizard',
            'Select default theme:'))
        self.progressLabel.setText(translate('OpenLP.FirstTimeWizard',
            'Starting configuration process...'))
        FirstTimeWizard.setButtonText(QtGui.QWizard.CustomButton1,
            translate('OpenLP.FirstTimeWizard', 'Finish'))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import SettingsTab, Receiver, translate
from openlp.core.lib.theme import ThemeLevel
from openlp.core.lib.ui import UiStrings, find_and_set_in_combo_box
from openlp.core.lib.settings import Settings

class ThemesTab(SettingsTab):
    """
    ThemesTab is the theme settings tab in the settings dialog.
    """
    def __init__(self, parent, mainwindow):
        self.mainwindow = mainwindow
        generalTranslated = translate('OpenLP.ThemesTab', 'Themes')
        SettingsTab.__init__(self, parent, u'Themes', generalTranslated)
        self.iconPath = u':/themes/theme_new.png'

    def setupUi(self):
        self.setObjectName(u'ThemesTab')
        SettingsTab.setupUi(self)
        self.GlobalGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.GlobalGroupBox.setObjectName(u'GlobalGroupBox')
        self.GlobalGroupBoxLayout = QtGui.QVBoxLayout(self.GlobalGroupBox)
        self.GlobalGroupBoxLayout.setObjectName(u'GlobalGroupBoxLayout')
        self.DefaultComboBox = QtGui.QComboBox(self.GlobalGroupBox)
        self.DefaultComboBox.setSizeAdjustPolicy(
            QtGui.QComboBox.AdjustToMinimumContentsLength)
        self.DefaultComboBox.setSizePolicy(
            QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        self.DefaultComboBox.setObjectName(u'DefaultComboBox')
        self.GlobalGroupBoxLayout.addWidget(self.DefaultComboBox)
        self.DefaultListView = QtGui.QLabel(self.GlobalGroupBox)
        self.DefaultListView.setObjectName(u'DefaultListView')
        self.GlobalGroupBoxLayout.addWidget(self.DefaultListView)
        self.leftLayout.addWidget(self.GlobalGroupBox)
        self.leftLayout.addStretch()
        self.LevelGroupBox = QtGui.QGroupBox(self.rightColumn)
        self.LevelGroupBox.setObjectName(u'LevelGroupBox')
        self.LevelLayout = QtGui.QFormLayout(self.LevelGroupBox)
        self.LevelLayout.setLabelAlignment(
            QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
        self.LevelLayout.setFormAlignment(
            QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
        self.LevelLayout.setObjectName(u'LevelLayout')
        self.SongLevelRadioButton = QtGui.QRadioButton(self.LevelGroupBox)
        self.SongLevelRadioButton.setObjectName(u'SongLevelRadioButton')
        self.SongLevelLabel = QtGui.QLabel(self.LevelGroupBox)
        self.SongLevelLabel.setObjectName(u'SongLevelLabel')
        self.LevelLayout.addRow(self.SongLevelRadioButton, self.SongLevelLabel)
        self.ServiceLevelRadioButton = QtGui.QRadioButton(self.LevelGroupBox)
        self.ServiceLevelRadioButton.setObjectName(u'ServiceLevelRadioButton')
        self.ServiceLevelLabel = QtGui.QLabel(self.LevelGroupBox)
        self.ServiceLevelLabel.setObjectName(u'ServiceLevelLabel')
        self.LevelLayout.addRow(self.ServiceLevelRadioButton,
            self.ServiceLevelLabel)
        self.GlobalLevelRadioButton = QtGui.QRadioButton(self.LevelGroupBox)
        self.GlobalLevelRadioButton.setObjectName(u'GlobalLevelRadioButton')
        self.GlobalLevelLabel = QtGui.QLabel(self.LevelGroupBox)
        self.GlobalLevelLabel.setObjectName(u'GlobalLevelLabel')
        self.LevelLayout.addRow(self.GlobalLevelRadioButton,
            self.GlobalLevelLabel)
        label_top_margin = (self.SongLevelRadioButton.sizeHint().height() -
            self.SongLevelLabel.sizeHint().height()) / 2
        for label in [self.SongLevelLabel, self.ServiceLevelLabel,
            self.GlobalLevelLabel]:
            rect = label.rect()
            rect.setTop(rect.top() + label_top_margin)
            label.setFrameRect(rect)
            label.setWordWrap(True)
        self.rightLayout.addWidget(self.LevelGroupBox)
        self.rightLayout.addStretch()
        QtCore.QObject.connect(self.SongLevelRadioButton,
            QtCore.SIGNAL(u'clicked()'), self.onSongLevelButtonClicked)
        QtCore.QObject.connect(self.ServiceLevelRadioButton,
            QtCore.SIGNAL(u'clicked()'), self.onServiceLevelButtonClicked)
        QtCore.QObject.connect(self.GlobalLevelRadioButton,
            QtCore.SIGNAL(u'clicked()'), self.onGlobalLevelButtonClicked)
        QtCore.QObject.connect(self.DefaultComboBox,
            QtCore.SIGNAL(u'activated(int)'), self.onDefaultComboBoxChanged)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'theme_update_list'), self.updateThemeList)

    def retranslateUi(self):
        self.tabTitleVisible = UiStrings().Themes
        self.GlobalGroupBox.setTitle(
            translate('OpenLP.ThemesTab', 'Global Theme'))
        self.LevelGroupBox.setTitle(
            translate('OpenLP.ThemesTab', 'Theme Level'))
        self.SongLevelRadioButton.setText(
            translate('OpenLP.ThemesTab', 'S&ong Level'))
        self.SongLevelLabel.setText(
            translate('OpenLP.ThemesTab', 'Use the theme from each song '
            'in the database. If a song doesn\'t have a theme associated with '
            'it, then use the service\'s theme. If the service doesn\'t have '
            'a theme, then use the global theme.'))
        self.ServiceLevelRadioButton.setText(
            translate('OpenLP.ThemesTab', '&Service Level'))
        self.ServiceLevelLabel.setText(
            translate('OpenLP.ThemesTab', 'Use the theme from the service, '
            'overriding any of the individual songs\' themes. If the '
            'service doesn\'t have a theme, then use the global theme.'))
        self.GlobalLevelRadioButton.setText(
            translate('OpenLP.ThemesTab', '&Global Level'))
        self.GlobalLevelLabel.setText(
            translate('OpenLP.ThemesTab', 'Use the global theme, overriding '
            'any themes associated with either the service or the songs.'))

    def load(self):
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        self.theme_level = settings.value(
            u'theme level', ThemeLevel.Song).toInt()[0]
        self.global_theme = unicode(settings.value(u'global theme').toString())
        settings.endGroup()
        if self.theme_level == ThemeLevel.Global:
            self.GlobalLevelRadioButton.setChecked(True)
        elif self.theme_level == ThemeLevel.Service:
            self.ServiceLevelRadioButton.setChecked(True)
        else:
            self.SongLevelRadioButton.setChecked(True)

    def save(self):
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        settings.setValue(u'theme level', QtCore.QVariant(self.theme_level))
        settings.setValue(u'global theme', QtCore.QVariant(self.global_theme))
        settings.endGroup()
        self.mainwindow.renderer.set_global_theme(self.global_theme)
        self.mainwindow.renderer.set_theme_level(self.theme_level)
        Receiver.send_message(u'theme_update_global', self.global_theme)

    def postSetUp(self):
        Receiver.send_message(u'theme_update_global', self.global_theme)

    def onSongLevelButtonClicked(self):
        self.theme_level = ThemeLevel.Song

    def onServiceLevelButtonClicked(self):
        self.theme_level = ThemeLevel.Service

    def onGlobalLevelButtonClicked(self):
        self.theme_level = ThemeLevel.Global

    def onDefaultComboBoxChanged(self, value):
        self.global_theme = unicode(self.DefaultComboBox.currentText())
        self.mainwindow.renderer.set_global_theme(self.global_theme)
        self.mainwindow.renderer.set_theme_level(self.theme_level)
        self.__previewGlobalTheme()

    def updateThemeList(self, theme_list):
        """
        Called from ThemeManager when the Themes have changed.

        ``theme_list``
            The list of available themes::

                [u'Bible Theme', u'Song Theme']
        """
        # Reload as may have been triggered by the ThemeManager.
        self.global_theme = unicode(Settings().value(
            self.settingsSection + u'/global theme',
            QtCore.QVariant(u'')).toString())
        self.DefaultComboBox.clear()
        self.DefaultComboBox.addItems(theme_list)
        find_and_set_in_combo_box(self.DefaultComboBox, self.global_theme)
        self.mainwindow.renderer.set_global_theme(self.global_theme)
        self.mainwindow.renderer.set_theme_level(self.theme_level)
        if self.global_theme is not u'':
            self.__previewGlobalTheme()

    def __previewGlobalTheme(self):
        """
        Utility method to update the global theme preview image.
        """
        image = self.mainwindow.themeManagerContents.getPreviewImage(
            self.global_theme)
        preview = QtGui.QPixmap(unicode(image))
        if not preview.isNull():
            preview = preview.scaled(300, 255, QtCore.Qt.KeepAspectRatio,
                QtCore.Qt.SmoothTransformation)
        self.DefaultListView.setPixmap(preview)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import chardet
import logging
import os
import re
import sqlite3
import time

from PyQt4 import QtCore
from sqlalchemy import Column, ForeignKey, or_, Table, types, func
from sqlalchemy.exc import OperationalError
from sqlalchemy.orm import class_mapper, mapper, relation
from sqlalchemy.orm.exc import UnmappedClassError

from openlp.core.lib import Receiver, translate
from openlp.core.lib.db import BaseModel, init_db, Manager
from openlp.core.lib.ui import critical_error_message_box
from openlp.core.utils import AppLocation, clean_filename
import upgrade

log = logging.getLogger(__name__)

RESERVED_CHARACTERS = u'\\.^$*+?{}[]()'


class BibleMeta(BaseModel):
    """
    Bible Meta Data
    """
    pass


class Book(BaseModel):
    """
    Song model
    """
    pass


class Verse(BaseModel):
    """
    Topic model
    """
    pass


def init_schema(url):
    """
    Setup a bible database connection and initialise the database schema.

    ``url``
        The database to setup.
    """
    session, metadata = init_db(url)

    meta_table = Table(u'metadata', metadata,
        Column(u'key', types.Unicode(255), primary_key=True, index=True),
        Column(u'value', types.Unicode(255)),
    )

    book_table = Table(u'book', metadata,
        Column(u'id', types.Integer, primary_key=True),
        Column(u'book_reference_id', types.Integer, index=True),
        Column(u'testament_reference_id', types.Integer),
        Column(u'name', types.Unicode(50), index=True),
    )
    verse_table = Table(u'verse', metadata,
        Column(u'id', types.Integer, primary_key=True, index=True),
        Column(u'book_id', types.Integer, ForeignKey(
            u'book.id'), index=True),
        Column(u'chapter', types.Integer, index=True),
        Column(u'verse', types.Integer, index=True),
        Column(u'text', types.UnicodeText, index=True),
    )

    try:
        class_mapper(BibleMeta)
    except UnmappedClassError:
        mapper(BibleMeta, meta_table)
    try:
        class_mapper(Book)
    except UnmappedClassError:
        mapper(Book, book_table,
            properties={'verses': relation(Verse, backref='book')})
    try:
        class_mapper(Verse)
    except UnmappedClassError:
        mapper(Verse, verse_table)

    metadata.create_all(checkfirst=True)
    return session


class BibleDB(QtCore.QObject, Manager):
    """
    This class represents a database-bound Bible. It is used as a base class
    for all the custom importers, so that the can implement their own import
    methods, but benefit from the database methods in here via inheritance,
    rather than depending on yet another object.
    """
    log.info(u'BibleDB loaded')

    def __init__(self, parent, **kwargs):
        """
        The constructor loads up the database and creates and initialises the
        tables if the database doesn't exist.

        **Required keyword arguments:**

        ``path``
            The path to the bible database file.

        ``name``
            The name of the database. This is also used as the file name for
            SQLite databases.
        """
        log.info(u'BibleDB loaded')
        QtCore.QObject.__init__(self)
        self.bible_plugin = parent
        if u'path' not in kwargs:
            raise KeyError(u'Missing keyword argument "path".')
        if u'name' not in kwargs and u'file' not in kwargs:
            raise KeyError(u'Missing keyword argument "name" or "file".')
        self.stop_import_flag = False
        if u'name' in kwargs:
            self.name = kwargs[u'name']
            if not isinstance(self.name, unicode):
                self.name = unicode(self.name, u'utf-8')
            self.file = clean_filename(self.name) + u'.sqlite'
        if u'file' in kwargs:
            self.file = kwargs[u'file']
        Manager.__init__(self, u'bibles', init_schema, self.file, upgrade)
        if u'file' in kwargs:
            self.get_name()
        if u'path' in kwargs:
            self.path = kwargs[u'path']
        self.wizard = None
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'openlp_stop_wizard'), self.stop_import)

    def stop_import(self):
        """
        Stops the import of the Bible.
        """
        log.debug(u'Stopping import')
        self.stop_import_flag = True

    def get_name(self):
        """
        Returns the version name of the Bible.
        """
        version_name = self.get_object(BibleMeta, u'name')
        self.name = version_name.value if version_name else None
        return self.name

    def register(self, wizard):
        """
        This method basically just initialialises the database. It is called
        from the Bible Manager when a Bible is imported. Descendant classes
        may want to override this method to supply their own custom
        initialisation as well.

        ``wizard``
            The actual Qt wizard form.
        """
        self.wizard = wizard
        return self.name

    def create_book(self, name, bk_ref_id, testament=1):
        """
        Add a book to the database.

        ``name``
            The name of the book.

        ``bk_ref_id``
            The book_reference_id from bibles_resources.sqlite of the book.

        ``testament``
            *Defaults to 1.* The testament_reference_id from
            bibles_resources.sqlite of the testament this book belongs to.
        """
        log.debug(u'BibleDB.create_book("%s", "%s")', name, bk_ref_id)
        book = Book.populate(name=name, book_reference_id=bk_ref_id,
            testament_reference_id=testament)
        self.save_object(book)
        return book

    def update_book(self, book):
        """
        Update a book in the database.

        ``book``
            The book object
        """
        log.debug(u'BibleDB.update_book("%s")', book.name)
        return self.save_object(book)

    def delete_book(self, db_book):
        """
        Delete a book from the database.

        ``db_book``
            The book object.
        """
        log.debug(u'BibleDB.delete_book("%s")', db_book.name)
        if self.delete_object(Book, db_book.id):
            return True
        return False

    def create_chapter(self, book_id, chapter, textlist):
        """
        Add a chapter and its verses to a book.

        ``book_id``
            The id of the book being appended.

        ``chapter``
            The chapter number.

        ``textlist``
            A dict of the verses to be inserted. The key is the verse number,
            and the value is the verse text.
        """
        log.debug(u'BibleDBcreate_chapter("%s", "%s")', book_id, chapter)
        # Text list has book and chapter as first two elements of the array.
        for verse_number, verse_text in textlist.iteritems():
            verse = Verse.populate(
                book_id=book_id,
                chapter=chapter,
                verse=verse_number,
                text=verse_text
            )
            self.session.add(verse)
        try:
            self.session.commit()
        except OperationalError:
            # Wait 10ms and try again.
            time.sleep(0.01)
            self.session.commit()

    def create_verse(self, book_id, chapter, verse, text):
        """
        Add a single verse to a chapter.

        ``book_id``
            The id of the book being appended.

        ``chapter``
            The chapter number.

        ``verse``
            The verse number.

        ``text``
            The verse text.
        """
        if not isinstance(text, unicode):
            details = chardet.detect(text)
            text = unicode(text, details[u'encoding'])
        verse = Verse.populate(
            book_id=book_id,
            chapter=chapter,
            verse=verse,
            text=text
        )
        self.session.add(verse)
        return verse

    def save_meta(self, key, value):
        """
        Utility method to save or update BibleMeta objects in a Bible database.

        ``key``
            The key for this instance.

        ``value``
            The value for this instance.
        """
        if not isinstance(value, unicode):
            value = unicode(value)
        log.debug(u'BibleDB.save_meta("%s/%s")', key, value)
        meta = self.get_object(BibleMeta, key)
        if meta:
            meta.value = value
            self.save_object(meta)
        else:
            self.save_object(BibleMeta.populate(key=key, value=value))

    def get_book(self, book):
        """
        Return a book object from the database.

        ``book``
            The name of the book to return.
        """
        log.debug(u'BibleDB.get_book("%s")', book)
        return self.get_object_filtered(Book, Book.name.like(book + u'%'))

    def get_books(self):
        """
        A wrapper so both local and web bibles have a get_books() method that
        manager can call. Used in the media manager advanced search tab.
        """
        log.debug(u'BibleDB.get_books()')
        return self.get_all_objects(Book, order_by_ref=Book.id)

    def get_book_by_book_ref_id(self, id):
        """
        Return a book object from the database.

        ``id``
            The reference id of the book to return.
        """
        log.debug(u'BibleDB.get_book_by_book_ref_id("%s")', id)
        return self.get_object_filtered(Book, Book.book_reference_id.like(id))

    def get_book_ref_id_by_name(self, book, maxbooks, language_id=None):
        log.debug(u'BibleDB.get_book_ref_id_by_name:("%s", "%s")', book,
            language_id)
        book_id = None
        if BiblesResourcesDB.get_book(book, True):
            book_temp = BiblesResourcesDB.get_book(book, True)
            book_id = book_temp[u'id']
        elif BiblesResourcesDB.get_alternative_book_name(book):
            book_id = BiblesResourcesDB.get_alternative_book_name(book)
        elif AlternativeBookNamesDB.get_book_reference_id(book):
            book_id = AlternativeBookNamesDB.get_book_reference_id(book)
        else:
            from openlp.plugins.bibles.forms import BookNameForm
            book_name = BookNameForm(self.wizard)
            if book_name.exec_(book, self.get_books(), maxbooks):
                book_id = book_name.book_id
            if book_id:
                AlternativeBookNamesDB.create_alternative_book_name(
                    book, book_id, language_id)
        return book_id

    def get_book_ref_id_by_localised_name(self, book,
        language_selection):
        """
        Return the id of a named book.

        ``book``
            The name of the book, according to the selected language.

        ``language_selection``
            The language selection the user has chosen in the settings
            section of the Bible.
        """
        log.debug(u'get_book_ref_id_by_localised_name("%s", "%s")',
            book, language_selection)
        from openlp.plugins.bibles.lib import LanguageSelection, \
            BibleStrings
        book_names = BibleStrings().BookNames
        # escape reserved characters
        book_escaped = book
        for character in RESERVED_CHARACTERS:
            book_escaped = book_escaped.replace(
                character, u'\\' + character)
        regex_book = re.compile(u'\s*%s\s*' % u'\s*'.join(
            book_escaped.split()), re.UNICODE | re.IGNORECASE)
        if language_selection == LanguageSelection.Bible:
            db_book = self.get_book(book)
            if db_book:
                return db_book.book_reference_id
        elif language_selection == LanguageSelection.Application:
            books = filter(lambda key:
                regex_book.match(unicode(book_names[key])), book_names.keys())
            books = filter(None, map(BiblesResourcesDB.get_book, books))
            for value in books:
                if self.get_book_by_book_ref_id(value[u'id']):
                    return value[u'id']
        elif language_selection == LanguageSelection.English:
            books = BiblesResourcesDB.get_books_like(book)
            if books:
                book_list = filter(
                    lambda value: regex_book.match(value[u'name']), books)
                if not book_list:
                    book_list = books
                for value in book_list:
                    if self.get_book_by_book_ref_id(value[u'id']):
                        return value[u'id']
        return False

    def get_verses(self, reference_list, show_error=True):
        """
        This is probably the most used function. It retrieves the list of
        verses based on the user's query.

        ``reference_list``
            This is the list of references the media manager item wants. It is
            a list of tuples, with the following format::

                (book_reference_id, chapter, start_verse, end_verse)

            Therefore, when you are looking for multiple items, simply break
            them up into references like this, bundle them into a list. This
            function then runs through the list, and returns an amalgamated
            list of ``Verse`` objects. For example::

                [(u'35', 1, 1, 1), (u'35', 2, 2, 3)]
        """
        log.debug(u'BibleDB.get_verses("%s")', reference_list)
        verse_list = []
        book_error = False
        for book_id, chapter, start_verse, end_verse in reference_list:
            db_book = self.get_book_by_book_ref_id(book_id)
            if db_book:
                book_id = db_book.book_reference_id
                log.debug(u'Book name corrected to "%s"', db_book.name)
                if end_verse == -1:
                    end_verse = self.get_verse_count(book_id, chapter)
                verses = self.session.query(Verse)\
                    .filter_by(book_id=db_book.id)\
                    .filter_by(chapter=chapter)\
                    .filter(Verse.verse >= start_verse)\
                    .filter(Verse.verse <= end_verse)\
                    .order_by(Verse.verse)\
                    .all()
                verse_list.extend(verses)
            else:
                log.debug(u'OpenLP failed to find book with id "%s"', book_id)
                book_error = True
        if book_error and show_error:
            critical_error_message_box(
                translate('BiblesPlugin', 'No Book Found'),
                translate('BiblesPlugin', 'No matching book '
                'could be found in this Bible. Check that you '
                'have spelled the name of the book correctly.'))
        return verse_list

    def verse_search(self, text):
        """
        Search for verses containing text ``text``.

        ``text``
            The text to search for. If the text contains commas, it will be
            split apart and OR'd on the list of values. If the text just
            contains spaces, it will split apart and AND'd on the list of
            values.
        """
        log.debug(u'BibleDB.verse_search("%s")', text)
        verses = self.session.query(Verse)
        if text.find(u',') > -1:
            keywords = \
                [u'%%%s%%' % keyword.strip() for keyword in text.split(u',')]
            or_clause = [Verse.text.like(keyword) for keyword in keywords]
            verses = verses.filter(or_(*or_clause))
        else:
            keywords = \
                [u'%%%s%%' % keyword.strip() for keyword in text.split(u' ')]
            for keyword in keywords:
                verses = verses.filter(Verse.text.like(keyword))
        verses = verses.all()
        return verses

    def get_chapter_count(self, book):
        """
        Return the number of chapters in a book.

        ``book``
            The book object to get the chapter count for.
        """
        log.debug(u'BibleDB.get_chapter_count("%s")', book.name)
        count = self.session.query(func.max(Verse.chapter)).join(Book).filter(
            Book.book_reference_id==book.book_reference_id).scalar()
        if not count:
            return 0
        return count

    def get_verse_count(self, book_ref_id, chapter):
        """
        Return the number of verses in a chapter.

        ``book_ref_id``
            The book reference id.

        ``chapter``
            The chapter to get the verse count for.
        """
        log.debug(u'BibleDB.get_verse_count("%s", "%s")', book_ref_id, chapter)
        count = self.session.query(func.max(Verse.verse)).join(Book)\
            .filter(Book.book_reference_id==book_ref_id)\
            .filter(Verse.chapter==chapter)\
            .scalar()
        if not count:
            return 0
        return count

    def get_language(self, bible_name=None):
        """
        If no language is given it calls a dialog window where the user could
        select the bible language.
        Return the language id of a bible.

        ``book``
            The language the bible is.
        """
        log.debug(u'BibleDB.get_language()')
        from openlp.plugins.bibles.forms import LanguageForm
        language = None
        language_form = LanguageForm(self.wizard)
        if language_form.exec_(bible_name):
            language = unicode(language_form.languageComboBox.currentText())
        if not language:
            return False
        language = BiblesResourcesDB.get_language(language)
        language_id = language[u'id']
        self.save_meta(u'language_id', language_id)
        return language_id

    def is_old_database(self):
        """
        Returns ``True`` if it is a bible database, which has been created
        prior to 1.9.6.
        """
        try:
            self.session.query(Book).all()
        except:
            return True
        return False

    def dump_bible(self):
        """
        Utility debugging method to dump the contents of a bible.
        """
        log.debug(u'.........Dumping Bible Database')
        log.debug(u'...............................Books ')
        books = self.session.query(Book).all()
        log.debug(books)
        log.debug(u'...............................Verses ')
        verses = self.session.query(Verse).all()
        log.debug(verses)


class BiblesResourcesDB(QtCore.QObject, Manager):
    """
    This class represents the database-bound Bible Resources. It provide
    some resources which are used in the Bibles plugin.
    A wrapper class around a small SQLite database which contains the download
    resources, a biblelist from the different download resources, the books,
    chapter counts and verse counts for the web download Bibles, a language
    reference, the testament reference and some alternative book names. This
    class contains a singleton "cursor" so that only one connection to the
    SQLite database is ever used.
    """
    cursor = None

    @staticmethod
    def get_cursor():
        """
        Return the cursor object. Instantiate one if it doesn't exist yet.
        """
        if BiblesResourcesDB.cursor is None:
            filepath = os.path.join(
                AppLocation.get_directory(AppLocation.PluginsDir), u'bibles',
                u'resources', u'bibles_resources.sqlite')
            conn = sqlite3.connect(filepath)
            BiblesResourcesDB.cursor = conn.cursor()
        return BiblesResourcesDB.cursor

    @staticmethod
    def run_sql(query, parameters=()):
        """
        Run an SQL query on the database, returning the results.

        ``query``
            The actual SQL query to run.

        ``parameters``
            Any variable parameters to add to the query.
        """
        cursor = BiblesResourcesDB.get_cursor()
        cursor.execute(query, parameters)
        return cursor.fetchall()

    @staticmethod
    def get_books():
        """
        Return a list of all the books of the Bible.
        """
        log.debug(u'BiblesResourcesDB.get_books()')
        books = BiblesResourcesDB.run_sql(u'SELECT id, testament_id, name, '
                u'abbreviation, chapters FROM book_reference ORDER BY id')
        return [{
            u'id': book[0],
            u'testament_id': book[1],
            u'name': unicode(book[2]),
            u'abbreviation': unicode(book[3]),
            u'chapters': book[4]
        } for book in books]

    @staticmethod
    def get_book(name, lower=False):
        """
        Return a book by name or abbreviation.

        ``name``
            The name or abbreviation of the book.

        ``lower``
            True if the comparsion should be only lowercase
        """
        log.debug(u'BiblesResourcesDB.get_book("%s")', name)
        if not isinstance(name, unicode):
            name = unicode(name)
        if lower:
            books = BiblesResourcesDB.run_sql(u'SELECT id, testament_id, name, '
                    u'abbreviation, chapters FROM book_reference WHERE '
                    u'LOWER(name) = ? OR LOWER(abbreviation) = ?',
                    (name.lower(), name.lower()))
        else:
            books = BiblesResourcesDB.run_sql(u'SELECT id, testament_id, name, '
                    u'abbreviation, chapters FROM book_reference WHERE name = ?'
                    u' OR abbreviation = ?', (name, name))
        if books:
            return {
                u'id': books[0][0],
                u'testament_id': books[0][1],
                u'name': unicode(books[0][2]),
                u'abbreviation': unicode(books[0][3]),
                u'chapters': books[0][4]
            }
        else:
            return None

    @staticmethod
    def get_books_like(string):
        """
        Return the books which include string.

        ``string``
            The string to search for in the book names or abbreviations.
        """
        log.debug(u'BiblesResourcesDB.get_book_like("%s")', string)
        if not isinstance(string, unicode):
            name = unicode(string)
        books = BiblesResourcesDB.run_sql(u'SELECT id, testament_id, name, '
                u'abbreviation, chapters FROM book_reference WHERE '
                u'LOWER(name) LIKE ? OR LOWER(abbreviation) LIKE ?',
                (u'%' + string.lower() + u'%', u'%' + string.lower() + u'%'))
        if books:
            return [{
                u'id': book[0],
                u'testament_id': book[1],
                u'name': unicode(book[2]),
                u'abbreviation': unicode(book[3]),
                u'chapters': book[4]
            } for book in books]
        else:
            return None

    @staticmethod
    def get_book_by_id(id):
        """
        Return a book by id.

        ``id``
            The id of the book.
        """
        log.debug(u'BiblesResourcesDB.get_book_by_id("%s")', id)
        if not isinstance(id, int):
            id = int(id)
        books = BiblesResourcesDB.run_sql(u'SELECT id, testament_id, name, '
                u'abbreviation, chapters FROM book_reference WHERE id = ?',
                (id, ))
        if books:
            return {
                u'id': books[0][0],
                u'testament_id': books[0][1],
                u'name': unicode(books[0][2]),
                u'abbreviation': unicode(books[0][3]),
                u'chapters': books[0][4]
            }
        else:
            return None

    @staticmethod
    def get_chapter(book_ref_id, chapter):
        """
        Return the chapter details for a specific chapter of a book.

        ``book_ref_id``
            The id of a book.

        ``chapter``
            The chapter number.
        """
        log.debug(u'BiblesResourcesDB.get_chapter("%s", "%s")', book_ref_id,
            chapter)
        if not isinstance(chapter, int):
            chapter = int(chapter)
        chapters = BiblesResourcesDB.run_sql(u'SELECT id, book_reference_id, '
            u'chapter, verse_count FROM chapters WHERE book_reference_id = ?',
            (book_ref_id,))
        try:
            return {
                u'id': chapters[chapter-1][0],
                u'book_reference_id': chapters[chapter-1][1],
                u'chapter': chapters[chapter-1][2],
                u'verse_count': chapters[chapter-1][3]
            }
        except (IndexError, TypeError):
            return None

    @staticmethod
    def get_chapter_count(book_ref_id):
        """
        Return the number of chapters in a book.

        ``book_ref_id``
            The id of the book.
        """
        log.debug(u'BiblesResourcesDB.get_chapter_count("%s")', book_ref_id)
        details = BiblesResourcesDB.get_book_by_id(book_ref_id)
        if details:
            return details[u'chapters']
        return 0

    @staticmethod
    def get_verse_count(book_ref_id, chapter):
        """
        Return the number of verses in a chapter.

        ``book``
            The id of the book.

        ``chapter``
            The number of the chapter.
        """
        log.debug(u'BiblesResourcesDB.get_verse_count("%s", "%s")', book_ref_id,
            chapter)
        details = BiblesResourcesDB.get_chapter(book_ref_id, chapter)
        if details:
            return details[u'verse_count']
        return 0

    @staticmethod
    def get_download_source(source):
        """
        Return a download_source_id by source.

        ``name``
            The name or abbreviation of the book.
        """
        log.debug(u'BiblesResourcesDB.get_download_source("%s")', source)
        if not isinstance(source, unicode):
            source = unicode(source)
        source = source.title()
        dl_source = BiblesResourcesDB.run_sql(u'SELECT id, source FROM '
                u'download_source WHERE source = ?', (source.lower(),))
        if dl_source:
            return {
                u'id': dl_source[0][0],
                u'source': dl_source[0][1]
            }
        else:
            return None

    @staticmethod
    def get_webbibles(source):
        """
        Return the bibles a webbible provide for download.

        ``source``
            The source of the webbible.
        """
        log.debug(u'BiblesResourcesDB.get_webbibles("%s")', source)
        if not isinstance(source,  unicode):
            source = unicode(source)
        source = BiblesResourcesDB.get_download_source(source)
        bibles = BiblesResourcesDB.run_sql(u'SELECT id, name, abbreviation, '
            u'language_id, download_source_id FROM webbibles WHERE '
            u'download_source_id = ?', (source[u'id'],))
        if bibles:
            return [{
                u'id': bible[0],
                u'name': bible[1],
                u'abbreviation': bible[2],
                u'language_id': bible[3],
                u'download_source_id': bible[4]
            } for bible in bibles]
        else:
            return None

    @staticmethod
    def get_webbible(abbreviation, source):
        """
        Return the bibles a webbible provide for download.

        ``abbreviation``
            The abbreviation of the webbible.

        ``source``
            The source of the webbible.
        """
        log.debug(u'BiblesResourcesDB.get_webbibles("%s", "%s")', abbreviation,
            source)
        if not isinstance(abbreviation, unicode):
            abbreviation = unicode(abbreviation)
        if not isinstance(source, unicode):
            source = unicode(source)
        source = BiblesResourcesDB.get_download_source(source)
        bible = BiblesResourcesDB.run_sql(u'SELECT id, name, abbreviation, '
            u'language_id, download_source_id FROM webbibles WHERE '
            u'download_source_id = ? AND abbreviation = ?', (source[u'id'],
            abbreviation))
        try:
            return {
                u'id': bible[0][0],
                u'name': bible[0][1],
                u'abbreviation': bible[0][2],
                u'language_id': bible[0][3],
                u'download_source_id': bible[0][4]
            }
        except (IndexError, TypeError):
            return None

    @staticmethod
    def get_alternative_book_name(name, language_id=None):
        """
        Return a book_reference_id if the name matches.

        ``name``
            The name to search the id.

        ``language_id``
            The language_id for which language should be searched
        """
        log.debug(u'BiblesResourcesDB.get_alternative_book_name("%s", "%s")',
            name, language_id)
        if language_id:
            books = BiblesResourcesDB.run_sql(u'SELECT book_reference_id, name '
                u'FROM alternative_book_names WHERE language_id = ? ORDER BY '
                u'id', (language_id, ))
        else:
            books = BiblesResourcesDB.run_sql(u'SELECT book_reference_id, name '
                u'FROM alternative_book_names ORDER BY id')
        for book in books:
            if book[1].lower() == name.lower():
                return book[0]
        return None

    @staticmethod
    def get_language_by_id(language_id):
        """
        Return a dict containing the language id, name and code by id.

        ``id``
            The id of the language in the database.
        """
        log.debug(u'BiblesResourcesDB.get_language_by_id(%d)', language_id)
        language = BiblesResourcesDB.run_sql(u'SELECT id, name, code FROM '
                u'language WHERE id = ?', (language_id,))
        if language:
            return {
                u'id': language[0][0],
                u'name': unicode(language[0][1]),
                u'code': unicode(language[0][2])
            }
        else:
            return None

    @staticmethod
    def get_language(name):
        """
        Return a dict containing the language id, name and code by name or
        abbreviation.

        ``name``
            The name or abbreviation of the language.
        """
        log.debug(u'BiblesResourcesDB.get_language("%s")', name)
        if not isinstance(name, unicode):
            name = unicode(name)
        language = BiblesResourcesDB.run_sql(u'SELECT id, name, code FROM '
                u'language WHERE name = ? OR code = ?', (name, name.lower()))
        if language:
            return {
                u'id': language[0][0],
                u'name': unicode(language[0][1]),
                u'code': unicode(language[0][2])
            }
        else:
            return None

    @staticmethod
    def get_languages():
        """
        Return a dict containing all languages with id, name and code.
        """
        log.debug(u'BiblesResourcesDB.get_languages()')
        languages = BiblesResourcesDB.run_sql(u'SELECT id, name, code FROM '
                u'language ORDER by name')
        if languages:
            return [{
                u'id': language[0],
                u'name': unicode(language[1]),
                u'code': unicode(language[2])
            } for language in languages]
        else:
            return None

    @staticmethod
    def get_testament_reference():
        """
        Return a list of all testaments and their id of the Bible.
        """
        log.debug(u'BiblesResourcesDB.get_testament_reference()')
        testaments = BiblesResourcesDB.run_sql(u'SELECT id, name FROM '
                u'testament_reference ORDER BY id')
        return [
            {
            u'id': testament[0],
            u'name': unicode(testament[1])
            }
            for testament in testaments
        ]


class AlternativeBookNamesDB(QtCore.QObject, Manager):
    """
    This class represents a database-bound alternative book names system.
    """
    cursor = None
    conn = None

    @staticmethod
    def get_cursor():
        """
        Return the cursor object. Instantiate one if it doesn't exist yet.
        If necessary loads up the database and creates the tables if the
        database doesn't exist.
        """
        if AlternativeBookNamesDB.cursor is None:
            filepath = os.path.join(
                AppLocation.get_directory(AppLocation.DataDir), u'bibles',
                    u'alternative_book_names.sqlite')
            if not os.path.exists(filepath):
                #create new DB, create table alternative_book_names
                AlternativeBookNamesDB.conn = sqlite3.connect(filepath)
                AlternativeBookNamesDB.conn.execute(u'CREATE TABLE '
                    u'alternative_book_names(id INTEGER NOT NULL, '
                    u'book_reference_id INTEGER, language_id INTEGER, name '
                    u'VARCHAR(50), PRIMARY KEY (id))')
            else:
                #use existing DB
                AlternativeBookNamesDB.conn = sqlite3.connect(filepath)
            AlternativeBookNamesDB.cursor = AlternativeBookNamesDB.conn.cursor()
        return AlternativeBookNamesDB.cursor

    @staticmethod
    def run_sql(query, parameters=(), commit=None):
        """
        Run an SQL query on the database, returning the results.

        ``query``
            The actual SQL query to run.

        ``parameters``
            Any variable parameters to add to the query

        ``commit``
            If a commit statement is necessary this should be True.
        """
        cursor = AlternativeBookNamesDB.get_cursor()
        cursor.execute(query, parameters)
        if commit:
            AlternativeBookNamesDB.conn.commit()
        return cursor.fetchall()

    @staticmethod
    def get_book_reference_id(name,  language_id=None):
        """
        Return a book_reference_id if the name matches.

        ``name``
            The name to search the id.

        ``language_id``
            The language_id for which language should be searched
        """
        log.debug(u'AlternativeBookNamesDB.get_book_reference_id("%s", "%s")',
            name, language_id)
        if language_id:
            books = AlternativeBookNamesDB.run_sql(u'SELECT book_reference_id, '
                u'name FROM alternative_book_names WHERE language_id = ?',
                (language_id, ))
        else:
            books = AlternativeBookNamesDB.run_sql(u'SELECT book_reference_id, '
                u'name FROM alternative_book_names')
        for book in books:
            if book[1].lower() == name.lower():
                return book[0]
        return None

    @staticmethod
    def create_alternative_book_name(name, book_reference_id, language_id):
        """
        Add an alternative book name to the database.

        ``name``
            The name of the alternative book name.

        ``book_reference_id``
            The book_reference_id of the book.

        ``language_id``
            The language to which the alternative book name belong.
        """
        log.debug(u'AlternativeBookNamesDB.create_alternative_book_name("%s", '
            '"%s", "%s"', name, book_reference_id, language_id)
        return AlternativeBookNamesDB.run_sql(u'INSERT INTO '
            u'alternative_book_names(book_reference_id, language_id, name) '
            u'VALUES (?, ?, ?)', (book_reference_id, language_id, name), True)


class OldBibleDB(QtCore.QObject, Manager):
    """
    This class conects to the old bible databases to reimport them to the new
    database scheme.
    """
    cursor = None

    def __init__(self, parent, **kwargs):
        """
        The constructor loads up the database and creates and initialises the
        tables if the database doesn't exist.

        **Required keyword arguments:**

        ``path``
            The path to the bible database file.

        ``name``
            The name of the database. This is also used as the file name for
            SQLite databases.
        """
        log.info(u'OldBibleDB loaded')
        QtCore.QObject.__init__(self)
        if u'path' not in kwargs:
            raise KeyError(u'Missing keyword argument "path".')
        if u'file' not in kwargs:
            raise KeyError(u'Missing keyword argument "file".')
        if u'path' in kwargs:
            self.path = kwargs[u'path']
        if u'file' in kwargs:
            self.file = kwargs[u'file']

    def get_cursor(self):
        """
        Return the cursor object. Instantiate one if it doesn't exist yet.
        """
        if self.cursor is None:
            filepath = os.path.join(self.path, self.file)
            self.connection = sqlite3.connect(filepath)
            self.cursor = self.connection.cursor()
        return self.cursor

    def run_sql(self, query, parameters=()):
        """
        Run an SQL query on the database, returning the results.

        ``query``
            The actual SQL query to run.

        ``parameters``
            Any variable parameters to add to the query.
        """
        cursor = self.get_cursor()
        cursor.execute(query, parameters)
        return cursor.fetchall()

    def get_name(self):
        """
        Returns the version name of the Bible.
        """
        version_name = self.run_sql(u'SELECT value FROM '
                u'metadata WHERE key = "name"')
        if version_name:
            self.name = version_name[0][0]
        else:
            self.name = None
        return self.name

    def get_metadata(self):
        """
        Returns the metadata of the Bible.
        """
        metadata = self.run_sql(u'SELECT key, value FROM metadata '
            u'ORDER BY rowid')
        if metadata:
            return [{
                u'key': unicode(meta[0]),
                u'value': unicode(meta[1])
            } for meta in metadata]
        else:
            return None

    def get_book(self, name):
        """
        Return a book by name or abbreviation.

        ``name``
            The name or abbreviation of the book.
        """
        if not isinstance(name, unicode):
            name = unicode(name)
        books = self.run_sql(u'SELECT id, testament_id, name, '
                u'abbreviation FROM book WHERE LOWER(name) = ? OR '
                u'LOWER(abbreviation) = ?', (name.lower(), name.lower()))
        if books:
            return {
                u'id': books[0][0],
                u'testament_id': books[0][1],
                u'name': unicode(books[0][2]),
                u'abbreviation': unicode(books[0][3])
            }
        else:
            return None

    def get_books(self):
        """
        Returns the books of the Bible.
        """
        books = self.run_sql(u'SELECT name, id FROM book ORDER BY id')
        if books:
            return [{
                u'name': unicode(book[0]),
                u'id':int(book[1])
            } for book in books]
        else:
            return None

    def get_verses(self, book_id):
        """
        Returns the verses of the Bible.
        """
        verses = self.run_sql(u'SELECT book_id, chapter, verse, text FROM '
            u'verse WHERE book_id = ? ORDER BY id', (book_id, ))
        if verses:
            return [{
                u'book_id': int(verse[0]),
                u'chapter': int(verse[1]),
                u'verse': int(verse[2]),
                u'text': unicode(verse[3])
            } for verse in verses]
        else:
            return None

    def close_connection(self):
        self.cursor.close()
        self.connection.close()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
import os
from tempfile import gettempdir
import sqlite3

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Plugin, StringContent, build_icon, translate, \
    Receiver
from openlp.core.lib.db import Manager
from openlp.core.lib.ui import UiStrings, create_action
from openlp.core.utils import get_filesystem_encoding
from openlp.core.utils.actions import ActionList
from openlp.plugins.songs.lib import clean_song, upgrade, SongMediaItem, \
    SongsTab
from openlp.plugins.songs.lib.db import init_schema, Song
from openlp.plugins.songs.lib.importer import SongFormat
from openlp.plugins.songs.lib.olpimport import OpenLPSongImport

log = logging.getLogger(__name__)

class SongsPlugin(Plugin):
    """
    This is the number 1 plugin, if importance were placed on any
    plugins. This plugin enables the user to create, edit and display
    songs. Songs are divided into verses, and the verse order can be
    specified. Authors, topics and song books can be assigned to songs
    as well.
    """
    log.info(u'Song Plugin loaded')

    def __init__(self, plugin_helpers):
        """
        Create and set up the Songs plugin.
        """
        Plugin.__init__(self, u'songs', plugin_helpers, SongMediaItem, SongsTab)
        self.manager = Manager(u'songs', init_schema, upgrade_mod=upgrade)
        self.weight = -10
        self.iconPath = u':/plugins/plugin_songs.png'
        self.icon = build_icon(self.iconPath)

    def checkPreConditions(self):
        return self.manager.session is not None

    def initialise(self):
        log.info(u'Songs Initialising')
        Plugin.initialise(self)
        self.songImportItem.setVisible(True)
        self.songExportItem.setVisible(True)
        self.toolsReindexItem.setVisible(True)
        action_list = ActionList.get_instance()
        action_list.add_action(self.songImportItem, unicode(UiStrings().Import))
        action_list.add_action(self.songExportItem, unicode(UiStrings().Export))
        action_list.add_action(self.toolsReindexItem,
            unicode(UiStrings().Tools))
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'servicemanager_new_service'),
            self.clearTemporarySongs)


    def addImportMenuItem(self, import_menu):
        """
        Give the Songs plugin the opportunity to add items to the
        **Import** menu.

        ``import_menu``
            The actual **Import** menu item, so that your actions can
            use it as their parent.
        """
        # Main song import menu item - will eventually be the only one
        self.songImportItem = create_action(import_menu, u'songImportItem',
            text=translate('SongsPlugin', '&Song'),
            tooltip=translate('SongsPlugin',
            'Import songs using the import wizard.'),
            triggers=self.onSongImportItemClicked)
        import_menu.addAction(self.songImportItem)

    def addExportMenuItem(self, export_menu):
        """
        Give the Songs plugin the opportunity to add items to the
        **Export** menu.

        ``export_menu``
            The actual **Export** menu item, so that your actions can
            use it as their parent.
        """
        # Main song import menu item - will eventually be the only one
        self.songExportItem = create_action(export_menu, u'songExportItem',
            text=translate('SongsPlugin', '&Song'),
            tooltip=translate('SongsPlugin',
            'Exports songs using the export wizard.'),
            triggers=self.onSongExportItemClicked)
        export_menu.addAction(self.songExportItem)

    def addToolsMenuItem(self, tools_menu):
        """
        Give the alerts plugin the opportunity to add items to the
        **Tools** menu.

        ``tools_menu``
            The actual **Tools** menu item, so that your actions can
            use it as their parent.
        """
        log.info(u'add tools menu')
        self.toolsReindexItem = create_action(tools_menu, u'toolsReindexItem',
            text=translate('SongsPlugin', '&Re-index Songs'),
            icon=u':/plugins/plugin_songs.png',
            statustip=translate('SongsPlugin',
            'Re-index the songs database to improve searching and ordering.'),
            visible=False, triggers=self.onToolsReindexItemTriggered)
        tools_menu.addAction(self.toolsReindexItem)

    def onToolsReindexItemTriggered(self):
        """
        Rebuild each song.
        """
        maxSongs = self.manager.get_object_count(Song)
        if maxSongs == 0:
            return
        progressDialog = QtGui.QProgressDialog(
            translate('SongsPlugin', 'Reindexing songs...'), UiStrings().Cancel,
            0, maxSongs, self.formParent)
        progressDialog.setWindowTitle(translate('SongsPlugin', 'Reindexing songs'))
        progressDialog.setWindowModality(QtCore.Qt.WindowModal)
        songs = self.manager.get_all_objects(Song)
        for number, song in enumerate(songs):
            clean_song(self.manager, song)
            progressDialog.setValue(number + 1)
        self.manager.save_objects(songs)
        self.mediaItem.onSearchTextButtonClicked()

    def onSongImportItemClicked(self):
        if self.mediaItem:
            self.mediaItem.onImportClick()

    def onSongExportItemClicked(self):
        if self.mediaItem:
            self.mediaItem.onExportClick()

    def about(self):
        return translate('SongsPlugin', '<strong>Songs Plugin</strong>'
            '<br />The songs plugin provides the ability to display and '
            'manage songs.')

    def usesTheme(self, theme):
        """
        Called to find out if the song plugin is currently using a theme.

        Returns True if the theme is being used, otherwise returns False.
        """
        if self.manager.get_all_objects(Song, Song.theme_name == theme):
            return True
        return False

    def renameTheme(self, oldTheme, newTheme):
        """
        Renames a theme the song plugin is using making the plugin use the new
        name.

        ``oldTheme``
            The name of the theme the plugin should stop using.

        ``newTheme``
            The new name the plugin should now use.
        """
        songsUsingTheme = self.manager.get_all_objects(Song,
            Song.theme_name == oldTheme)
        for song in songsUsingTheme:
            song.theme_name = newTheme
            self.manager.save_object(song)

    def importSongs(self, format, **kwargs):
        class_ = SongFormat.get(format, u'class')
        kwargs[u'plugin'] = self
        importer = class_(self.manager, **kwargs)
        importer.register(self.mediaItem.importWizard)
        return importer

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('SongsPlugin', 'Song', 'name singular'),
            u'plural': translate('SongsPlugin', 'Songs', 'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('SongsPlugin', 'Songs', 'container title')
        }
        # Middle Header Bar
        tooltips = {
            u'load': u'',
            u'import': u'',
            u'new': translate('SongsPlugin', 'Add a new song.'),
            u'edit': translate('SongsPlugin', 'Edit the selected song.'),
            u'delete': translate('SongsPlugin', 'Delete the selected song.'),
            u'preview': translate('SongsPlugin', 'Preview the selected song.'),
            u'live': translate('SongsPlugin', 'Send the selected song live.'),
            u'service': translate('SongsPlugin',
                'Add the selected song to the service.')
        }
        self.setPluginUiTextStrings(tooltips)

    def firstTime(self):
        """
        If the first time wizard has run, this function is run to import all the
        new songs into the database.
        """
        Receiver.send_message(u'openlp_process_events')
        self.onToolsReindexItemTriggered()
        Receiver.send_message(u'openlp_process_events')
        db_dir = unicode(os.path.join(
            unicode(gettempdir(), get_filesystem_encoding()), u'openlp'))
        if not os.path.exists(db_dir):
            return
        song_dbs = []
        song_count = 0
        for sfile in os.listdir(db_dir):
            if sfile.startswith(u'songs_') and sfile.endswith(u'.sqlite'):
                Receiver.send_message(u'openlp_process_events')
                song_dbs.append(os.path.join(db_dir, sfile))
                song_count += self._countSongs(os.path.join(db_dir, sfile))
        if not song_dbs:
            return
        Receiver.send_message(u'openlp_process_events')
        progress = QtGui.QProgressDialog(self.formParent)
        progress.setWindowModality(QtCore.Qt.WindowModal)
        progress.setWindowTitle(translate('OpenLP.Ui', 'Importing Songs'))
        progress.setLabelText(translate('OpenLP.Ui', 'Starting import...'))
        progress.setCancelButton(None)
        progress.setRange(0, song_count)
        progress.setMinimumDuration(0)
        progress.forceShow()
        Receiver.send_message(u'openlp_process_events')
        for db in song_dbs:
            importer = OpenLPSongImport(self.manager, filename=db)
            importer.doImport(progress)
            Receiver.send_message(u'openlp_process_events')
        progress.setValue(song_count)
        self.mediaItem.onSearchTextButtonClicked()

    def finalise(self):
        """
        Time to tidy up on exit
        """
        log.info(u'Songs Finalising')
        self.clearTemporarySongs()
        # Clean up files and connections
        self.manager.finalise()
        self.songImportItem.setVisible(False)
        self.songExportItem.setVisible(False)
        self.toolsReindexItem.setVisible(False)
        action_list = ActionList.get_instance()
        action_list.remove_action(self.songImportItem,
            unicode(UiStrings().Import))
        action_list.remove_action(self.songExportItem,
            unicode(UiStrings().Export))
        action_list.remove_action(self.toolsReindexItem,
            unicode(UiStrings().Tools))
        Plugin.finalise(self)

    def clearTemporarySongs(self):
        # Remove temporary songs
        songs = self.manager.get_all_objects(Song, Song.temporary == True)
        for song in songs:
            self.manager.delete_object(Song, song.id)

    def _countSongs(self, db_file):
        connection = sqlite3.connect(db_file)
        cursor = connection.cursor()
        cursor.execute(u'SELECT COUNT(id) AS song_count FROM songs')
        song_count = cursor.fetchone()[0]
        connection.close()
        try:
            song_count = int(song_count)
        except (TypeError, ValueError):
            song_count = 0
        return song_count



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtGui

from openlp.core.lib import translate, build_icon
from openlp.core.lib.ui import create_button_box, create_button


class Ui_ServiceItemEditDialog(object):
    def setupUi(self, serviceItemEditDialog):
        serviceItemEditDialog.setObjectName(u'serviceItemEditDialog')
        serviceItemEditDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.dialogLayout = QtGui.QGridLayout(serviceItemEditDialog)
        self.dialogLayout.setContentsMargins(8, 8, 8, 8)
        self.dialogLayout.setSpacing(8)
        self.dialogLayout.setObjectName(u'dialogLayout')
        self.listWidget = QtGui.QListWidget(serviceItemEditDialog)
        self.listWidget.setAlternatingRowColors(True)
        self.listWidget.setObjectName(u'listWidget')
        self.dialogLayout.addWidget(self.listWidget, 0, 0)
        self.buttonLayout = QtGui.QVBoxLayout()
        self.buttonLayout.setObjectName(u'buttonLayout')
        self.deleteButton = create_button(serviceItemEditDialog,
            u'deleteButton', role=u'delete',
            click=serviceItemEditDialog.onDeleteButtonClicked)
        self.buttonLayout.addWidget(self.deleteButton)
        self.buttonLayout.addStretch()
        self.upButton = create_button(serviceItemEditDialog, u'upButton',
            role=u'up', click=serviceItemEditDialog.onUpButtonClicked)
        self.downButton = create_button(serviceItemEditDialog, u'downButton',
            role=u'down', click=serviceItemEditDialog.onDownButtonClicked)
        self.buttonLayout.addWidget(self.upButton)
        self.buttonLayout.addWidget(self.downButton)
        self.dialogLayout.addLayout(self.buttonLayout, 0, 1)
        self.buttonBox = create_button_box(serviceItemEditDialog, u'buttonBox',
            [u'cancel', u'save'])
        self.dialogLayout.addWidget(self.buttonBox, 1, 0, 1, 2)
        self.retranslateUi(serviceItemEditDialog)

    def retranslateUi(self, serviceItemEditDialog):
        serviceItemEditDialog.setWindowTitle(
            translate('OpenLP.ServiceItemEditForm', 'Reorder Service Item'))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`zionworximport` module provides the functionality for importing
ZionWorx songs into the OpenLP database.
"""
import csv
import logging

from openlp.core.lib import translate
from openlp.plugins.songs.lib.songimport import SongImport

log = logging.getLogger(__name__)

# Used to strip control chars (except 10=LF, 13=CR)
CONTROL_CHARS_MAP = dict.fromkeys(range(10) + [11, 12] + range(14,32) + [127])

class ZionWorxImport(SongImport):
    """
    The :class:`ZionWorxImport` class provides the ability to import songs
    from ZionWorx, via a dump of the ZionWorx database to a CSV file.

    ZionWorx song database fields:

    * ``SongNum`` Song ID. (Discarded by importer)
    * ``Title1`` Main Title.
    * ``Title2`` Alternate Title.
    * ``Lyrics`` Song verses, separated by blank lines.
    * ``Writer`` Song author(s).
    * ``Copyright`` Copyright information
    * ``Keywords`` (Discarded by importer)
    * ``DefaultStyle`` (Discarded by importer)

    ZionWorx has no native export function; it uses the proprietary TurboDB
    database engine. The TurboDB vendor, dataWeb, provides tools which can
    export TurboDB tables to other formats, such as freeware console tool
    TurboDB Data Exchange which is available for Windows and Linux. This command
    exports the ZionWorx songs table to a CSV file:

    ``tdbdatax MainTable.dat songstable.csv -fsdf -s, -qd``

    * -f  Table format: ``sdf`` denotes text file.
    * -s  Separator character between fields.
    * -q  Quote character surrounding fields. ``d`` denotes double-quote.

    CSV format expected by importer:

    * Field separator character is comma ``,``
    * Fields surrounded by double-quotes ``"``. This enables fields (such as
      Lyrics) to include new-lines and commas. Double-quotes within a field
      are denoted by two double-quotes ``""``
    * Note: This is the default format of the Python ``csv`` module.

    """
    def doImport(self):
        """
        Receive a CSV file (from a ZionWorx database dump) to import.
        """
        with open(self.importSource, 'rb') as songs_file:
            field_names = [u'SongNum', u'Title1', u'Title2', u'Lyrics',
                u'Writer', u'Copyright', u'Keywords', u'DefaultStyle']
            songs_reader = csv.DictReader(songs_file, field_names)
            try:
                records = list(songs_reader)
            except csv.Error, e:
                self.logError(unicode(translate('SongsPlugin.ZionWorxImport',
                    'Error reading CSV file.')),
                    unicode(translate('SongsPlugin.ZionWorxImport',
                    'Line %d: %s')) % (songs_reader.line_num, e))
                return
            num_records = len(records)
            log.info(u'%s records found in CSV file' % num_records)
            self.importWizard.progressBar.setMaximum(num_records)
            for index, record in enumerate(records, 1):
                if self.stopImportFlag:
                    return
                self.setDefaults()
                try:
                    self.title = self._decode(record[u'Title1'])
                    if record[u'Title2']:
                        self.alternateTitle = self._decode(record[u'Title2'])
                    self.parseAuthor(self._decode(record[u'Writer']))
                    self.addCopyright(self._decode(record[u'Copyright']))
                    lyrics = self._decode(record[u'Lyrics'])
                except UnicodeDecodeError, e:
                    self.logError(unicode(translate(
                        'SongsPlugin.ZionWorxImport', 'Record %d' % index)),
                        unicode(translate('SongsPlugin.ZionWorxImport',
                        'Decoding error: %s')) % e)
                    continue
                except TypeError, e:
                    self.logError(unicode(translate(
                        'SongsPlugin.ZionWorxImport', 'File not valid ZionWorx '
                        'CSV format.')), u'TypeError: %s' % e)
                    return
                verse = u''
                for line in lyrics.splitlines():
                    if line and not line.isspace():
                        verse += line + u'\n'
                    elif verse:
                        self.addVerse(verse)
                        verse = u''
                if verse:
                    self.addVerse(verse)
                title = self.title
                if not self.finish():
                    self.logError(unicode(translate(
                        'SongsPlugin.ZionWorxImport', 'Record %d')) % index
                        + (u': "' + title + u'"' if title else u''))

    def _decode(self, str):
        """
        Decodes CSV input to unicode, stripping all control characters (except
        new lines).
        """
        # This encoding choice seems OK. ZionWorx has no option for setting the
        # encoding for its songs, so we assume encoding is always the same.
        return unicode(str, u'cp1252').translate(CONTROL_CHARS_MAP)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from openlp.core.lib import Plugin, StringContent, build_icon, translate
from openlp.core.lib.db import Manager
from openlp.plugins.custom.lib import CustomMediaItem, CustomTab
from openlp.plugins.custom.lib.db import CustomSlide, init_schema

log = logging.getLogger(__name__)

class CustomPlugin(Plugin):
    """
    This plugin enables the user to create, edit and display
    custom slide shows. Custom shows are divided into slides.
    Each show is able to have it's own theme.
    Custom shows are designed to replace the use of songs where
    the songs plugin has become restrictive. Examples could be
    Welcome slides, Bible Reading information, Orders of service.
    """
    log.info(u'Custom Plugin loaded')

    def __init__(self, plugin_helpers):
        Plugin.__init__(self, u'custom', plugin_helpers,
            CustomMediaItem, CustomTab)
        self.weight = -5
        self.manager = Manager(u'custom', init_schema)
        self.iconPath = u':/plugins/plugin_custom.png'
        self.icon = build_icon(self.iconPath)

    def about(self):
        about_text = translate('CustomPlugin', '<strong>Custom Slide Plugin'
            '</strong><br />The custom slide plugin provides the ability to '
            'set up custom text slides that can be displayed on the screen '
            'the same way songs are. This plugin provides greater freedom '
            'over the songs plugin.')
        return about_text

    def usesTheme(self, theme):
        """
        Called to find out if the custom plugin is currently using a theme.

        Returns True if the theme is being used, otherwise returns False.
        """
        if self.manager.get_all_objects(CustomSlide,
            CustomSlide.theme_name == theme):
            return True
        return False

    def renameTheme(self, oldTheme, newTheme):
        """
        Renames a theme the custom plugin is using making the plugin use the
        new name.

        ``oldTheme``
            The name of the theme the plugin should stop using.

        ``newTheme``
            The new name the plugin should now use.
        """
        customsUsingTheme = self.manager.get_all_objects(CustomSlide,
            CustomSlide.theme_name == oldTheme)
        for custom in customsUsingTheme:
            custom.theme_name = newTheme
            self.manager.save_object(custom)

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('CustomPlugin', 'Custom Slide',
                                   'name singular'),
            u'plural': translate('CustomPlugin', 'Custom Slides',
                                 'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('CustomPlugin', 'Custom Slides',
                'container title')
        }
        # Middle Header Bar
        tooltips = {
            u'load': translate('CustomPlugin', 'Load a new custom slide.'),
            u'import': translate('CustomPlugin', 'Import a custom slide.'),
            u'new': translate('CustomPlugin', 'Add a new custom slide.'),
            u'edit': translate('CustomPlugin',
                'Edit the selected custom slide.'),
            u'delete': translate('CustomPlugin',
                'Delete the selected custom slide.'),
            u'preview': translate('CustomPlugin',
                'Preview the selected custom slide.'),
            u'live': translate('CustomPlugin',
                'Send the selected custom slide live.'),
            u'service': translate('CustomPlugin',
                'Add the selected custom slide to the service.')
        }
        self.setPluginUiTextStrings(tooltips)

    def finalise(self):
        """
        Time to tidy up on exit
        """
        log.info(u'Custom Finalising')
        self.manager.finalise()
        Plugin.finalise(self)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`cvsbible` modules provides a facility to import bibles from a set of
CSV files.

The module expects two mandatory files containing the books and the verses.

The format of the books file is:

    <book_id>,<testament_id>,<book_name>,<book_abbreviation>

    For example

        1,1,Genesis,Gen
        2,1,Exodus,Exod
        ...
        40,2,Matthew,Matt

There are two acceptable formats of the verses file.  They are:

    <book_id>,<chapter_number>,<verse_number>,<verse_text>
    or
    <book_name>,<chapter_number>,<verse_number>,<verse_text>

    For example:

        1,1,1,"In the beginning God created the heaven and the earth."
        or
        "Genesis",1,2,"And the earth was without form, and void; and...."

All CSV files are expected to use a comma (',') as the delimeter and double
quotes ('"') as the quote symbol.
"""
import logging
import chardet
import csv

from openlp.core.lib import Receiver, translate
from openlp.plugins.bibles.lib.db import BibleDB, BiblesResourcesDB

log = logging.getLogger(__name__)

class CSVBible(BibleDB):
    """
    This class provides a specialisation for importing of CSV Bibles.
    """
    log.info(u'CSVBible loaded')

    def __init__(self, parent, **kwargs):
        """
        Loads a Bible from a set of CSV files.
        This class assumes the files contain all the information and
        a clean bible is being loaded.
        """
        log.info(self.__class__.__name__)
        BibleDB.__init__(self, parent, **kwargs)
        self.booksfile = kwargs[u'booksfile']
        self.versesfile = kwargs[u'versefile']

    def do_import(self, bible_name=None):
        """
        Import the bible books and verses.
        """
        self.wizard.progressBar.setValue(0)
        self.wizard.progressBar.setMinimum(0)
        self.wizard.progressBar.setMaximum(66)
        success = True
        language_id = self.get_language(bible_name)
        if not language_id:
            log.exception(u'Importing books from "%s" failed' % self.filename)
            return False
        books_file = None
        book_list = {}
        # Populate the Tables
        try:
            details = get_file_encoding(self.booksfile)
            books_file = open(self.booksfile, 'r')
            if not books_file.read(3) == '\xEF\xBB\xBF':
                # no BOM was found
                books_file.seek(0)
            books_reader = csv.reader(books_file, delimiter=',', quotechar='"')
            for line in books_reader:
                if self.stop_import_flag:
                    break
                self.wizard.incrementProgressBar(unicode(
                    translate('BiblesPlugin.CSVBible',
                    'Importing books... %s')) %
                    unicode(line[2], details['encoding']))
                book_ref_id = self.get_book_ref_id_by_name(
                    unicode(line[2], details['encoding']), 67, language_id)
                if not book_ref_id:
                    log.exception(u'Importing books from "%s" '\
                        'failed' % self.booksfile)
                    return False
                book_details = BiblesResourcesDB.get_book_by_id(book_ref_id)
                self.create_book(unicode(line[2], details['encoding']),
                    book_ref_id, book_details[u'testament_id'])
                book_list[int(line[0])] = unicode(line[2], details['encoding'])
            Receiver.send_message(u'openlp_process_events')
        except (IOError, IndexError):
            log.exception(u'Loading books from file failed')
            success = False
        finally:
            if books_file:
                books_file.close()
        if self.stop_import_flag or not success:
            return False
        self.wizard.progressBar.setValue(0)
        self.wizard.progressBar.setMaximum(67)
        verse_file = None
        try:
            book_ptr = None
            details = get_file_encoding(self.versesfile)
            verse_file = open(self.versesfile, 'rb')
            if not verse_file.read(3) == '\xEF\xBB\xBF':
                # no BOM was found
                verse_file.seek(0)
            verse_reader = csv.reader(verse_file, delimiter=',', quotechar='"')
            for line in verse_reader:
                if self.stop_import_flag:
                    break
                try:
                    line_book = book_list[int(line[0])]
                except ValueError:
                    line_book = unicode(line[0], details['encoding'])
                if book_ptr != line_book:
                    book = self.get_book(line_book)
                    book_ptr = book.name
                    self.wizard.incrementProgressBar(unicode(translate(
                        'BiblesPlugin.CSVBible', 'Importing verses from %s...',
                        'Importing verses from <book name>...')) % book.name)
                    self.session.commit()
                try:
                    verse_text = unicode(line[3], details['encoding'])
                except UnicodeError:
                    verse_text = unicode(line[3], u'cp1252')
                self.create_verse(book.id, line[1], line[2], verse_text)
            self.wizard.incrementProgressBar(translate('BiblesPlugin.CSVBible',
                'Importing verses... done.'))
            Receiver.send_message(u'openlp_process_events')
            self.session.commit()
        except IOError:
            log.exception(u'Loading verses from file failed')
            success = False
        finally:
            if verse_file:
                verse_file.close()
        if self.stop_import_flag:
            return False
        else:
            return success

def get_file_encoding(filename):
    """
    Utility function to get the file encoding.
    """
    detect_file = None
    try:
        detect_file = open(filename, 'r')
        details = chardet.detect(detect_file.read(1024))
    except IOError:
        log.exception(u'Error detecting file encoding')
    finally:
        if detect_file:
            detect_file.close()
    return details
